package tbox;

import javax.swing.JDialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

import java.awt.event.ActionEvent;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;

@SuppressWarnings("serial")
public class AddDialog extends JDialog implements ActionListener, FocusListener {
	private JPanel addPanel = null;
	private JButton yesButton = null;
	private JButton noButton = null;
	private boolean answer = false;
	private JTextField nameField = new JTextField();
	private JTextArea description = new JTextArea();
	JRadioButton containerButton = new JRadioButton();
	String projectKey = null;
	private JComboBox types = new JComboBox();
	private String pName = null;
	private String pLocation = null;
	/**
	 * Set up the mySQL database access to the treasurebox database.
	 */
	static String url = "jdbc:mysql://localhost:3306/treasurebox";
	static String driver = "com.mysql.jdbc.Driver";
	static String user = "treasureboxuser";
	static String pass = "treasure";

	public boolean getAnswer() {
		return answer;
	}

	public String getItemName() {
		return nameField.getText();
	}

	public String getDescription() {
		return description.getText();
	}

	public String getIsContainer() {
		if (containerButton.isSelected())
			return "Y";
		else
			return "N";
	}

	public String getTypeX() {
		return types.getSelectedItem().toString();
	}

	public void focusGained(FocusEvent e) {
		if (nameField.getText().equals("Enter the Item Name Here"))
			nameField.setText("");
	}

	public void focusLost(FocusEvent e) {
	}

	public void actionPerformed(ActionEvent e) {
		/**
		 * The user has requested the item to be added to the database. The code here verifies that the item values do
		 * not contain errors. If an error is found a message panel is displayed indicating the problem. Once all fields
		 * have been validated control is passed back to the calling method which performs the insert.
		 */
		if (yesButton == e.getSource()) {
			boolean errorFound = false;
			/**
			 * The item name field is checked to make sure that it has a least one character and at most 25 characters.
			 */
			if (nameField.getText().length() < 1 || nameField.getText().length() > 25) {
				JOptionPane.showMessageDialog(null, "Item Name '" + nameField.getText()
						+ "' must be at least 1 character long and at most 25 characters long.", "Cannot Edit Item "
						+ pName, JOptionPane.ERROR_MESSAGE);
				errorFound = true;
			}
			/**
			 * The item name field is checked to make sure that it contains only alpla-numeric (A-Z, a-z, 0-9), space and
			 * the following characters (-#@$:.%). If other than these are found an error message is displayed.
			 */
			for (int i = 0; i < nameField.getText().length(); i++) {
				if (!Character.isLetterOrDigit(nameField.getText().charAt(i))
						&& !Character.isWhitespace(nameField.getText().charAt(i))
						&& !Character.toString(nameField.getText().charAt(i)).equals("-")
						&& !Character.toString(nameField.getText().charAt(i)).equals("#")
						&& !Character.toString(nameField.getText().charAt(i)).equals("@")
						&& !Character.toString(nameField.getText().charAt(i)).equals("$")
						&& !Character.toString(nameField.getText().charAt(i)).equals(":")
						&& !Character.toString(nameField.getText().charAt(i)).equals(".")
						&& !Character.toString(nameField.getText().charAt(i)).equals("%")) {
					JOptionPane.showMessageDialog(null,
							"Name field must be alphanumeric A-Z, a-z, 0-9, space or the following characters: -#@$%.:",
							"Enter a proper Name ", JOptionPane.ERROR_MESSAGE);
					errorFound = true;
				}
			}
			/**
			 * The item description is checked to make sure that it has a least one character and at most 1000
			 * characters.
			 */
			if (description.getText().length() < 1 || description.getText().length() > 1000) {
				JOptionPane.showMessageDialog(null, "Description '" + description.getText()
						+ "' must be at least 1 character long and at most 1000 characters long.", "Cannot Edit Item "
						+ pName, JOptionPane.ERROR_MESSAGE);
				errorFound = true;
			}
			/**
			 * The item name field is checked to make sure the user did not enter an existing node name at the same
			 * level. If the user enters a duplicate node name an error message is displayed and the add dialog does not
			 * close.
			 */
			String query = null;
			Connection conn = null;
			try {
				Class.forName(driver).newInstance();
				conn = DriverManager.getConnection(url, user, pass);
				Statement stmt = conn.createStatement();
				ResultSet rs;
				if (pLocation.equals(" ")) {
					query = "SELECT name FROM items WHERE location = '" + pName + "' AND name = '"
							+ nameField.getText() + "' AND projectKey = " + projectKey + ";";
				} else {
					query = "SELECT name FROM items WHERE location = '" + pLocation + "|" + pName + "' AND name = '"
							+ nameField.getText() + "' AND projectKey = " + projectKey + ";";
				}
				System.out.println(query);
				rs = stmt.executeQuery(query);
				while (rs.next()) {
//					String itemName = rs.getString("name");
					JOptionPane.showMessageDialog(null, "Item Name '" + nameField.getText()
							+ "' already exists under '" + pName + "'. Choose another name!", "Cannot Add Item in "
							+ pName, JOptionPane.ERROR_MESSAGE);
					errorFound = true;
					break;
				}
				conn.close();
			} catch (Exception e1) {
				System.err.println("Got an exception! ");
				System.err.println(e1.getMessage());
			}
			/**
			 * The type field is checked to insure that it does not contain the system keyword 'ROOT'.
			 */
			if (types.getSelectedItem().toString().toUpperCase().equals("ROOT")) {
				JOptionPane.showMessageDialog(null, "Item Name '" + nameField.getText()
						+ "' cannot have a TYPE of 'ROOT'. Choose or enter another type!", "Cannot Add Item "
						+ nameField.getText(), JOptionPane.ERROR_MESSAGE);
				errorFound = true;
			}
			/**
			 * The item type is checked to make sure that it has a least one character and at most 15 characters.
			 */
			if (types.getSelectedItem().toString().length() < 1 || types.getSelectedItem().toString().length() > 15) {
				JOptionPane.showMessageDialog(null, "Item Type '" + types.getSelectedItem().toString()
						+ "' must be at least 1 character long and at most 15 characters long.", "Cannot Edit Item "
						+ pName, JOptionPane.ERROR_MESSAGE);
				errorFound = true;
			}
			/**
			 * The item type is checked to make sure that it contains only alpla-numeric (A-Z, a-z, 0-9), space and the
			 * following characters (-#@$:.%). If other than these are found an error message is displayed.
			 */
			for (int i = 0; i < types.getSelectedItem().toString().length(); i++) {
				if (!Character.isLetterOrDigit(types.getSelectedItem().toString().charAt(i))
						&& !Character.isWhitespace(types.getSelectedItem().toString().charAt(i))
						&& !Character.toString(types.getSelectedItem().toString().charAt(i)).equals("-")
						&& !Character.toString(types.getSelectedItem().toString().charAt(i)).equals("#")
						&& !Character.toString(types.getSelectedItem().toString().charAt(i)).equals("@")
						&& !Character.toString(types.getSelectedItem().toString().charAt(i)).equals("$")
						&& !Character.toString(types.getSelectedItem().toString().charAt(i)).equals(":")
						&& !Character.toString(types.getSelectedItem().toString().charAt(i)).equals(".")
						&& !Character.toString(types.getSelectedItem().toString().charAt(i)).equals("%")) {
					JOptionPane.showMessageDialog(null,
							"Item Type must be alphanumeric A-Z, a-z, 0-9, space or the following characters: -#@$%.:",
							"Enter a proper Type ", JOptionPane.ERROR_MESSAGE);
					errorFound = true;
				}
			}
			/**
			 * If the name and type fields are valid the dialog is closed and the item is added. If an error is found
			 * the add dialog does not close.
			 */
			if (!errorFound) {
				System.err.println("User chose yes.");
				answer = true;
				setVisible(false);
			}
			/**
			 * If the user chose cancel exit the add panel.
			 */
		} else if (noButton == e.getSource()) {
			System.err.println("User chose no.");
			answer = false;
			setVisible(false);
		}
	}

	/**
	 * The constructor for the class. Sets up all the components in the panel.
	 */
	public AddDialog(JFrame frame, boolean modal, String myMessage, String parentName, String parentLocation,
			String pKey) {
		super(frame, "Add a New Item in " + parentName, modal);
		// Use a black line border
		Border blackline;
		blackline = BorderFactory.createLineBorder(Color.black);
		this.setMinimumSize(new Dimension(400, 325));
		this.setResizable(false);
		pName = parentName;
		projectKey = pKey;
		pLocation = parentLocation;
		// Item name
		nameField = new JTextField(25);
		nameField.setBorder(blackline);
		nameField.setFont(new Font("TimesNewRoman", Font.PLAIN, 12));
		nameField.setText("Enter the Item Name Here");
		nameField.addFocusListener(this);
		// Create the Container radio buttons.
		containerButton = new JRadioButton("Container");
		containerButton.setMnemonic(KeyEvent.VK_C);
		containerButton.setActionCommand("container");
		containerButton.setSelected(true);

		JRadioButton notContainerButton = new JRadioButton("Not a Container");
		notContainerButton.setMnemonic(KeyEvent.VK_N);
		notContainerButton.setActionCommand("notContainer");

		// Group the Container radio buttons.
		ButtonGroup group = new ButtonGroup();
		group.add(containerButton);
		group.add(notContainerButton);

		// Register a listener for the radio buttons.
		containerButton.addActionListener(null);
		notContainerButton.addActionListener(null);
		// Place label, and two radio buttons into radioPanel
		JPanel radioPanel = new JPanel(new FlowLayout());
		radioPanel.setBorder(blackline);
		radioPanel.add(new JLabel("Is this Item a Container?"));
		radioPanel.add(containerButton);
		radioPanel.add(notContainerButton);

		// Place Label, Item Name and Radio Panel into the top panel.
		JPanel topPanel = new JPanel(new BorderLayout());
		topPanel.setBorder(blackline);
		JLabel title = new JLabel("Enter Item Information Below", SwingConstants.LEFT);
		title.setFont(new Font("TimesNewRoman", Font.BOLD, 14));
		topPanel.add(title, BorderLayout.NORTH);
		topPanel.add(nameField, BorderLayout.WEST);
		topPanel.add(radioPanel, BorderLayout.EAST);

		// Place Item Description into descPanel
		JPanel descPanel = new JPanel();
		descPanel.setBorder(blackline);
		description = new JTextArea("Enter Item Description Here", 12, 30);
		description.setFont(new Font("TimesNewRoman", Font.PLAIN, 12));
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setBorder(blackline);
		JScrollPane itemInfoScrollPane = new JScrollPane(description);
		description.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				if (description.getText().equals("Enter Item Description Here"))
					description.setText("");
			}

			public void focusLost(FocusEvent e) {
			}
		});
		// Place Label and Type ComboBox into typePanel
		JPanel typePanel = new JPanel();
		typePanel.setBorder(blackline);
		typePanel.add(new JLabel("Enter Item Type or \"None\""));

		types = new JComboBox();
		types.setEditable(true);
		typePanel.add(types);
		/**
		 * Get the list of types for the project from the database. The type of 'ROOT' is removed from the list.
		 */
		Connection conn = null;
		try {
			Class.forName(driver).newInstance();
			conn = DriverManager.getConnection(url, user, pass);
			Statement stmt = conn.createStatement();
			ResultSet rs;

			String query = "SELECT DISTINCT(type) FROM items WHERE projectKey = " + projectKey + ";";
			rs = stmt.executeQuery(query);
			while (rs.next()) {
				String type = rs.getString("type");
				if (!type.equals("ROOT"))
					types.addItem(type);
			}
			conn.close();
		} catch (Exception e1) {
			System.err.println("Got an exception! ");
			System.err.println(e1.getMessage());
		}

		// Place Description and Type into descPanel
		descPanel.add(itemInfoScrollPane);
		descPanel.add(typePanel);

		// Place Label from caller, yes and no buttons into groupPanel
		JPanel groupPanel = new JPanel();
		groupPanel.add(new JLabel(myMessage));
		yesButton = new JButton("Yes");
		yesButton.addActionListener(this);
		groupPanel.add(yesButton);
		noButton = new JButton("Cancel");
		noButton.addActionListener(this);
		groupPanel.add(noButton);

		// Add the three panels to the main panel myPanel
		addPanel = new JPanel(new BorderLayout());
		addPanel.add(topPanel, BorderLayout.NORTH);
		addPanel.add(descPanel, BorderLayout.CENTER);
		addPanel.add(groupPanel, BorderLayout.SOUTH);

		getContentPane().add(addPanel);

		pack();
		setLocationRelativeTo(frame);
		setVisible(true);
	}

}
