package tbox;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * The Finder class is a component in the TreasureBox application which controls a horizontal split pane containing the
 * item tree and image view panels. Tree maintenance is handled by the three buttons Add, Edit and Delete directly
 * beneath the tree view. Selecting an item from the tree loads all images for an item from the database and displays
 * the first imageImage if one exists. Image management has four buttons for image maintenance (Add Before, Replace, Add
 * After and Delete) and five buttons for image manipulation (Next, Previous, Zoom In, Zoom Out and Rotate Right).
 */

public class Finder extends JPanel {
	private static final long serialVersionUID = 1L;
	public JSplitPane splitPane;
	private JLabel picture;
	public JTree tree;
	public DefaultMutableTreeNode treeTop = null;
	public DefaultTreeModel model = new DefaultTreeModel(treeTop);

	JPanel itemButtonPanel = new JPanel();
	String projectKey = new String();
	String projectName = new String();
	String projectDescription = new String();
	String selectedKey = new String();
	String selectedName = new String();
	String selectedType = new String();
	String selectedDescription = new String();
	String selectedLocation = new String();
	int selectedLevel = 0;
	String selectedContainer = new String();
	String selectedImage = new String();
	String replacedImage = new String();
	String rootName = new String();
	String updateStatement = new String();
	String[] itemPhotos = new String[25];
	static String newline = System.getProperty("line.separator");
	static String userDir = System.getProperty("user.home");
	String lastDirectory = new String(userDir);
	int photoCount = 0;
	int currentPhoto = 0;
	int zoomWidth = 0, zoomHeight = 0;
	int rotateDegrees = 0;
	int rotateMode = 0;
	boolean maintenanceMode = true;
	boolean logFlag = false;
	JTextArea itemInfoText = new JTextArea();

	FileWriter fstream;
	static BufferedWriter msgWriter;

	JScrollPane itemInfoScrollPane = new JScrollPane(itemInfoText);
	JScrollPane treeView = new JScrollPane();
	JScrollPane pictureScrollPane = new JScrollPane(picture);

	JButton replaceButton = new JButton();
	JButton nextButton, previousButton, addAfterButton, addBeforeButton, deleteButton;
	JButton plusButton, minusButton;
	JButton rotateRightButton;
	JButton addItemButton = new JButton();
	JButton editItemButton = new JButton();
	JButton deleteItemButton = new JButton();

	JFrame frame = new JFrame();
	TreeSelectionListener treeSelListener;
	// SQL Components
	Connection conn1 = null;
	Connection conn2 = null;
	int updateStatus = 0;
	/**
	 * Set up the mySQL database access to the treasurebox database.
	 */
	String url = "jdbc:mysql://localhost:3306/treasurebox";
	String driver = "com.mysql.jdbc.Driver";
	String user = "treasureboxuser";
	String pass = "treasure";

	static String pathSeperator = System.getProperty("path.separator");
	/**
	 * Determine the install path where the jar file resides. The command returns the path and name of the executable
	 * jar file. The name is removed to set the install path alone.
	 */
	StringBuffer OSPath = new StringBuffer(ProjectDialog.class.getProtectionDomain().getCodeSource().getLocation()
			.getPath());
	int pathEnd = OSPath.lastIndexOf("/");
	String installPath = OSPath.substring(0, pathEnd + 1);

	/**
	 * The constructor for the split pane component containing the item tree and item viewer panel.
	 */
	public Finder() {
		projectKey = null;
		tree = new JTree();
		/**
		 * Prompts the user for a project, retrieves all project records from the database and builds the node structure
		 * to support the tree displayed in the panel.
		 */
		createNodes(true);
		/**
		 * Create and display the tree. Allows one selection at a time.
		 */
		model = new DefaultTreeModel(treeTop, true);
		tree = new JTree(model);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setEditable(false);
		expandAll(tree, true);
		tree.validate();
		/**
		 * The Tree Selection Listener executes each time the user selects a new item in the item tree. Once the item is
		 * selected a SQL query returns the item's values and another SQL query obtains any images that have been
		 * assigned to the item. If any images are found the first image is displayed.
		 */
		treeSelListener = new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				zoomWidth = 0;
				zoomHeight = 0;
				rotateDegrees = 0;
				rotateMode = 0;
				TreePath tp = e.getPath();
				int pathCnt = tp.getPathCount() - 1;
				StringBuffer sbWhere = new StringBuffer();
				// Build Location value and size to compare
				for (int i = 0; i < pathCnt; i++) {
					sbWhere.append(tp.getPathComponent(i));
					if (i < pathCnt - 1)
						sbWhere.append("|");
				}
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
				if (node == null)
					return;
				Object nodeInfo = node.getUserObject();
				/**
				 * Create and execute an SQL query to retrieve the selected item information from the items table.
				 */
				String query = "SELECT itemKey, name, description, location, level, type, container FROM items WHERE name = '"
						+ nodeInfo.toString()
						+ "' AND location = '"
						+ sbWhere
						+ "' AND projectKey = "
						+ projectKey
						+ ";";
				if (logFlag)
					logMessage(3, query);
				Connection conn = null;
				try {
					Class.forName(driver).newInstance();
					conn = DriverManager.getConnection(url, user, pass);
					Statement stmt = conn.createStatement();
					ResultSet rs;

					rs = stmt.executeQuery(query);
					while (rs.next()) {
						selectedKey = rs.getString("itemKey");
						selectedName = rs.getString("name");
						selectedDescription = rs.getString("description");
						selectedLocation = rs.getString("location");
						selectedLevel = rs.getInt("level");
						selectedType = rs.getString("type");
						selectedContainer = rs.getString("container");
					}
					/**
					 * Create and execute an SQL query to get the image names and order from the images table for the
					 * selected item.
					 */
					photoCount = 0;
					currentPhoto = 0;
					query = "SELECT imageName, imageOrder FROM images WHERE itemKey = " + selectedKey
							+ " ORDER BY imageOrder;";
					rs = stmt.executeQuery(query);
					while (rs.next()) {
						itemPhotos[photoCount++] = rs.getString("imageName");
					}
					conn.close();
				} catch (Exception e1) {
					if (logFlag)
						logMessage(2,
								"Error when selecting image name and order from images in valueChanged(TreeSelectionEvent)");
				}
				/**
				 * Determine the screen width in characters and adjust the itemInfoText width and height based on the
				 * screen width.
				 */
				int screenWidth = (splitPane.getWidth() - splitPane.getDividerLocation()) / 13;
				itemInfoText.setColumns(screenWidth);
				itemInfoText.setRows(3);
				int screenHeight = (splitPane.getHeight());
				/**
				 * Allow 40 pixels for the Add, Edit and Delete buttons if in maintenance mode.
				 */
				if (maintenanceMode)
					treeView.setPreferredSize((new Dimension(220, screenHeight - 60)));
				else
					treeView.setPreferredSize((new Dimension(220, screenHeight - 10)));
				treeView.repaint();
				/**
				 * Based on the number of images and the maintenance mode set the visibility for the three item
				 * management buttons and the nine image controls.
				 */
				if (photoCount == 0) {
					itemInfoText.setText(selectedName + "          " + "(No Photos Added)   Item Type = "
							+ selectedType + "\nDescription:  " + selectedDescription);
					replaceButton.setText("Add...");
					updateImage(installPath + "/" + "blank.jpg");
					if (maintenanceMode)
						replaceButton.setVisible(true);
					else
						replaceButton.setVisible(false);
					addAfterButton.setVisible(false);
					addBeforeButton.setVisible(false);
					nextButton.setVisible(false);
					rotateRightButton.setVisible(false);
					previousButton.setVisible(false);
					deleteButton.setVisible(false);
					plusButton.setVisible(false);
					minusButton.setVisible(false);
				} else {
					selectedImage = itemPhotos[0];
					itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
							+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
							+ "\nDescription:  " + selectedDescription);
					itemInfoText.setCaretPosition(0);
					replaceButton.setText("Replace...");
					if (maintenanceMode)
						replaceButton.setVisible(true);
					else
						replaceButton.setVisible(false);
					selectedImage = itemPhotos[0];
					currentPhoto = 0;
					updateImage(installPath + projectName + "/" + selectedImage);
					if (photoCount == 1) {
						if (maintenanceMode) {
							addAfterButton.setVisible(true);
							addBeforeButton.setVisible(true);
							deleteButton.setVisible(true);
						} else {
							addAfterButton.setVisible(false);
							addBeforeButton.setVisible(false);
							deleteButton.setVisible(false);
						}
						nextButton.setVisible(false);
						previousButton.setVisible(false);
						rotateRightButton.setVisible(true);
						plusButton.setVisible(true);
						minusButton.setVisible(true);
					} else {
						if (maintenanceMode) {
							addAfterButton.setVisible(true);
							addBeforeButton.setVisible(true);
							deleteButton.setVisible(true);
						} else {
							addAfterButton.setVisible(false);
							addBeforeButton.setVisible(false);
							deleteButton.setVisible(false);
						}
						nextButton.setVisible(true);
						previousButton.setVisible(true);
						rotateRightButton.setVisible(true);
						plusButton.setVisible(true);
						minusButton.setVisible(true);
					}
				}
				if (maintenanceMode) {
					if (selectedContainer.equals("N"))
						addItemButton.setVisible(false);
					else
						addItemButton.setVisible(true);
					editItemButton.setVisible(true);
					deleteItemButton.setVisible(true);
					itemButtonPanel.setVisible(true);
				} else {
					addItemButton.setVisible(false);
					editItemButton.setVisible(false);
					deleteItemButton.setVisible(false);
					itemButtonPanel.setVisible(false);
				}
			}
		};
		tree.addTreeSelectionListener(treeSelListener);

		// Create the scroll pane and add the tree to it.
		treeView = new JScrollPane(tree);
		JPanel treePanel = new JPanel();
		treePanel.setBackground(Color.red);
		treePanel.setLayout(new FlowLayout());
		treePanel.setPreferredSize(new Dimension(220, 500));
		treePanel.add(treeView);

		addItemButton = new JButton("Add...");
		editItemButton = new JButton("Edit...");
		deleteItemButton = new JButton("Delete...");
		if (maintenanceMode) {
			addItemButton.setVisible(true);
			editItemButton.setVisible(true);
			deleteItemButton.setVisible(true);
			itemButtonPanel.setVisible(true);
		} else {
			addItemButton.setVisible(false);
			editItemButton.setVisible(false);
			deleteItemButton.setVisible(false);
			itemButtonPanel.setVisible(false);
		}

		itemButtonPanel.add(addItemButton);
		itemButtonPanel.add(editItemButton);
		itemButtonPanel.add(deleteItemButton);

		treePanel.add(itemButtonPanel);

		picture = new JLabel();
		picture.setFont(picture.getFont().deriveFont(Font.ITALIC));
		picture.setHorizontalAlignment(JLabel.CENTER);
		JPanel itemPanel = new JPanel();
		JPanel cntls = new JPanel();
		JPanel itemHeaderPanel = new JPanel();

		itemInfoText = new JTextArea("Item Name", 3, 40);
		itemInfoText.setFont(new Font("TimesNewRoman", Font.PLAIN, 12));
		itemInfoText.setLineWrap(true);
		itemInfoText.setWrapStyleWord(true);
		itemInfoScrollPane = new JScrollPane(itemInfoText);
		itemInfoText.setEditable(false);
		pictureScrollPane = new JScrollPane(picture);
		itemHeaderPanel.setLayout(new FlowLayout());
		itemHeaderPanel.setBackground(Color.blue);
		cntls.setLayout(new FlowLayout());
		cntls.setBackground(Color.blue);
		itemPanel.setLayout(new BorderLayout());
		// Define the Item Buttons
		nextButton = new JButton("Next");
		previousButton = new JButton("Previous");
		addBeforeButton = new JButton("< Insert...");
		addAfterButton = new JButton("Insert >...");
		replaceButton = new JButton("Replace...");
		deleteButton = new JButton("Delete");
		plusButton = new JButton("+");
		minusButton = new JButton("-");
		rotateRightButton = new JButton("Rotate Right");

		itemHeaderPanel.add(itemInfoScrollPane);

		// Add the buttons to the button controls panel
		cntls.add(nextButton);
		cntls.add(previousButton);
		cntls.add(addBeforeButton);
		cntls.add(replaceButton);
		cntls.add(addAfterButton);
		cntls.add(deleteButton);
		cntls.add(plusButton);
		cntls.add(minusButton);
		cntls.add(rotateRightButton);

		// Add the Item Header Panel, Picture Panel and Picture Controls to the panel
		itemPanel.add(itemHeaderPanel, BorderLayout.NORTH);
		itemPanel.add(pictureScrollPane, BorderLayout.CENTER);
		itemPanel.add(cntls, BorderLayout.SOUTH);
		itemPanel.validate();

		/**
		 * Action Listener for the Item Tree Panel. Handles the Add, Edit and Delete Buttons for selected item.
		 */
		ActionListener alItem = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DefaultMutableTreeNode child = null;
				DefaultMutableTreeNode parent = null;

				String command = e.getActionCommand();
				/**
				 * The item Add button displays the AddDialog to the user to add a new item under the selected item.
				 * Once the item fields have been verified and the user selects yes to add the item to the database the
				 * tree display is updated and a SQL Insert statement is created to insert the new item. A subsequent
				 * query is then executed to retrieve the newly generated item key.
				 */
				if (command.equals("add")) {
					// Present the user with a modal Dialog to determine the
					// item values.
					AddDialog ad = new AddDialog(frame, true, "Add Item to " + selectedName + "? ", selectedName,
							selectedLocation, projectKey);
					// If the user did not cancel the transaction, try to insert
					// the new item.
					if (ad.getAnswer()) {
						updateStatement = new String();
						// Set the Location
						String newLocation = new String();
						if (selectedLocation.equals(" ")) {
							newLocation = selectedName;
						} else {
							newLocation = selectedLocation + "|" + selectedName;
						}
						int levelCount = 1;
						int fromIndex = 0;
						while (newLocation.indexOf("|", fromIndex) != -1) {
							levelCount++;
							fromIndex = newLocation.indexOf("|", fromIndex) + 1;
						}
						// Set the selected values
						selectedName = ad.getItemName();
						selectedDescription = ad.getDescription();
						selectedType = ad.getTypeX();
						selectedContainer = ad.getIsContainer();
						selectedImage = "blank.jpg";
						selectedLocation = newLocation;
						photoCount = 0;
						TreePath tp = tree.getSelectionPath();
						parent = (DefaultMutableTreeNode) (tp.getLastPathComponent());
						if (parent == null)
							return;
						int itemOrderCnt = parent.getChildCount();
						child = new DefaultMutableTreeNode(selectedName);
						model.insertNodeInto(child, parent, parent.getChildCount());

						if (selectedContainer.equals("Y")) {
							child.setAllowsChildren(true);
						} else {
							child.setAllowsChildren(false);
						}
						tree.repaint();
						TreePath path = new TreePath(child.getPath());
						tree.setSelectionPath(path);
						updateImage(installPath + "blank.jpg");
						replaceButton.setText("Add...");
						photoCount = 0;
						if (maintenanceMode)
							replaceButton.setVisible(true);
						else
							replaceButton.setVisible(false);
						addAfterButton.setVisible(false);
						addBeforeButton.setVisible(false);
						deleteButton.setVisible(false);
						rotateRightButton.setVisible(false);
						nextButton.setVisible(false);
						previousButton.setVisible(false);
						plusButton.setVisible(false);
						minusButton.setVisible(false);
						itemInfoText.setText(selectedName + "                     ("
								+ "No Photos Added)   Item Type = " + selectedType + "\nDescription:  "
								+ selectedDescription);
						// SQL execution
						updateStatement = "INSERT INTO items (name, projectKey, itemOrder, description, container,type, location, locationsize, level)"
								+ " VALUES(? , "
								+ projectKey
								+ ", "
								+ itemOrderCnt
								+ ",? ,'"
								+ ad.getIsContainer()
								+ "', ?, '" + newLocation + "', " + newLocation.length() + ", " + levelCount + ");";
						if (logFlag)
							logMessage(3, updateStatement);
						Connection conn = null;
						try {
							Class.forName(driver).newInstance();
							conn = DriverManager.getConnection(url, user, pass);
							PreparedStatement pStmt = conn.prepareStatement(updateStatement);
							pStmt.setString(1, ad.getItemName());
							pStmt.setString(2, ad.getDescription());
							pStmt.setString(3, ad.getTypeX());
							updateStatus = pStmt.executeUpdate();
							String query = "SELECT itemKey FROM items WHERE location = '" + newLocation
									+ "' AND name = '" + selectedName + "' AND projectKey = " + projectKey + ";";
							ResultSet rSet = pStmt.executeQuery(query);
							while (rSet.next()) {
								selectedKey = rSet.getString("itemkey");
							}
							conn.close();
						} catch (Exception e1) {
							if (logFlag)
								logMessage(2, "Error when retrieving itemKey from items in Add item of Finder.");
						}
					}
					/**
					 * The item Edit button displays the EditDialog to the user to edit the selected item. Once the item
					 * fields have been verified and the user selects yes to edit the item in the database the display
					 * is updated and a SQL Update statement is created to update the item.
					 * 
					 * If the selected item is the project ROOT and the project name is changed the project's image
					 * directory will also be changed(renamed). If the ROOT item name or description are changed a
					 * corresponding update will be made to the project name and description in the project table so
					 * that the change will be displayed in the Project panel.
					 * 
					 * If the item name field has changed then any child items will have their location fields updated
					 * to reflect the new item name. The location field is a concatenation of item names from the
					 * selected item's parent up to the ROOT or project name.
					 * 
					 * If the user makes the item "Not a Container" (radio button) then any items under the selected
					 * item are removed as well as any images associated with those deleted items.
					 */
				} else if (command.equals("edit")) {
					// Hold the name to determine if the user changes this
					// value.
					String holdName = selectedName;
					String holdContainer = selectedContainer;
					EditDialog ed = new EditDialog(frame, true, "Modify Item " + selectedName + "? ", selectedName,
							selectedLocation, selectedLevel, selectedDescription, selectedType, selectedContainer,
							projectKey);
					if (ed.getAnswer()) {
						// If the user selected OK then get the edit fields from
						// the Edit Dialog screen.
						selectedName = ed.getItemName();
						selectedDescription = ed.getDescription();
						selectedType = ed.getTypeX();
						selectedContainer = ed.getIsContainer();

						DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree
								.getLastSelectedPathComponent();
						if (selectedNode != null) {
							if (!selectedName.equals(holdName))
								selectedNode.setUserObject(selectedName);

							if (selectedContainer.equals("Y")) {
								/**
								 * If this is the root item and the name is changed, rename the project directory to
								 * match.
								 * 
								 * If the current item is a container and the item name has changed, update all the
								 * locations that contain the old name with the new name.
								 */
								if (!selectedName.equals(holdName)) {
									if (selectedType.equals("ROOT")) {
										File oldProject = new File(installPath + holdName);
										File newProject = new File(installPath + selectedName);
										boolean success = oldProject.renameTo(newProject);
										if (!success) {
											if (logFlag)
												logMessage(2, "Unable to rename directory '" + holdName + "' to '"
														+ selectedName + "'");
										}
										projectName = selectedName;
									}
									String query;
									// SQL execution
									try {
										Class.forName(driver).newInstance();
										conn1 = DriverManager.getConnection(url, user, pass);
										conn2 = DriverManager.getConnection(url, user, pass);
										Statement stmt1 = conn1.createStatement();
										Statement stmt2 = conn2.createStatement();
										ResultSet rs;
										/**
										 * Select all items which are under the selected item and have the old item name
										 * in its location. If this is a ROOT item place the old name at the beginning
										 * of the location else place the old name after the selected location.
										 */
										if (selectedLocation.equals(" "))
											query = "SELECT itemKey, location FROM items WHERE location LIKE '"
													+ holdName + "%' AND projectKey = " + projectKey + ";";
										else
											query = "SELECT itemKey, location FROM items WHERE location LIKE '"
													+ selectedLocation + "|" + holdName + "%' AND projectKey = "
													+ projectKey + ";";
										if (logFlag)
											logMessage(3, query);
										rs = stmt1.executeQuery(query);
										while (rs.next()) {
											String itemKey = rs.getString("itemkey");
											String location = rs.getString("location");
											int locSize = selectedLocation.length() + holdName.length() + 1;
											String newLocation;
											/**
											 * If this is the ROOT item append the new name to beginning of the location
											 */
											if (selectedLocation.equals(" "))
												newLocation = selectedName + location.substring(holdName.length());
											/**
											 * If this is the child of the selected item append the new name to the end
											 * of the location
											 */
											else if ((selectedLocation + "|" + holdName).length() == location.length())
												newLocation = selectedLocation + "|" + selectedName;
											/**
											 * All other items will have the new name replaced within the location.
											 */
											else
												newLocation = selectedLocation + "|" + selectedName
														+ location.substring(locSize);
											locSize = newLocation.length();
											updateStatement = "UPDATE items SET location = '" + newLocation
													+ "', locationSize = " + locSize + " WHERE itemKey = " + itemKey
													+ ";";
											if (logFlag)
												logMessage(3, updateStatement);
											updateStatus = stmt2.executeUpdate(updateStatement);
										}
										conn1.close();
										conn2.close();
									} catch (Exception e1) {
										if (logFlag)
											logMessage(2,
													"Error when inserting or updating to image table when Add or Replace image.");
									}
								}
								selectedNode.setAllowsChildren(true);
								if (maintenanceMode)
									addItemButton.setVisible(true);
								else
									addItemButton.setVisible(false);
								/**
								 * If the item container has been changed from 'Y' to 'N' any children items under the
								 * selected item are deleted along with any images associated with these deleted items.
								 * Images are deleted internally by the database using the cascaded delete in the MySQL
								 * image table definition.
								 */
							} else if (!selectedContainer.equals(holdContainer)) {
								selectedNode.setAllowsChildren(false);
								addItemButton.setVisible(false);
								// SQL execution
								Connection conn = null;
								try {
									Class.forName(driver).newInstance();
									conn = DriverManager.getConnection(url, user, pass);
									Statement stmt = conn.createStatement();
									// Delete all items which were under the selected item.
									updateStatement = "DELETE FROM items WHERE location LIKE '" + selectedLocation
											+ "|" + selectedName + "%' AND projectKey = " + projectKey + ";";
									if (logFlag)
										logMessage(3, updateStatement);
									updateStatus = stmt.executeUpdate(updateStatement);
									conn.close();
								} catch (Exception e1) {
									if (logFlag)
										logMessage(2,
												"Error when deleting all items under the selected item if Container is changed to 'N' in Edit item.");
								}
							}
							model.reload(selectedNode);
							tree.repaint();
						}
						updateStatement = new String();
						if (photoCount == 0) {
							itemInfoText.setText(selectedName + "                     ("
									+ "No Photos Added)   Item Type = " + selectedType + "\nDescription:  "
									+ selectedDescription);
						} else {
							itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
									+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
									+ "\nDescription:  " + selectedDescription);
						}
						itemInfoText.setCaretPosition(0);
						/**
						 * Create the prepared statement to update the item to the values specified in the edit dialog.
						 */
						updateStatement = "UPDATE items SET name = ?, description = ?, container = '"
								+ ed.getIsContainer() + "', type = '" + ed.getType() + "' WHERE itemKey = "
								+ selectedKey + ";";
						if (logFlag)
							logMessage(3, updateStatement);
						Connection conn = null;
						try {
							Class.forName(driver).newInstance();
							conn = DriverManager.getConnection(url, user, pass);
							PreparedStatement pStmt = conn.prepareStatement(updateStatement);
							pStmt.setString(1, selectedName);
							pStmt.setString(2, selectedDescription);
							updateStatus = pStmt.executeUpdate();
							/**
							 * If this is a ROOT item update the project name and description in the project table
							 */
							if (ed.getType().equals("ROOT")) {
								updateStatement = "UPDATE projects SET name = ?, description = ? WHERE projectKey = "
										+ projectKey + ";";
								if (logFlag)
									logMessage(3, updateStatement);
								pStmt = conn.prepareStatement(updateStatement);
								pStmt.setString(1, selectedName);
								pStmt.setString(2, selectedDescription);
								updateStatus = pStmt.executeUpdate();
							}
							conn.close();
						} catch (Exception e1) {
							if (logFlag)
								logMessage(2,
										"Error when updating the project table in Edit item when ROOT is changed.");
						}
					}
					/**
					 * The item Delete button displays a warning dialog to the user to verify the deletion of the
					 * selected item and all items which have been defined under the selected item. Once the user has
					 * verified the request the display is updated and the item and any children items are deleted from
					 * the database. Since the image table has a cascaded delete any images for the deleted items are
					 * removed from the database as well. The image deletions are handled by the database system.
					 */
				} else if (command.equals("delete")) {
					int s1 = JOptionPane.showConfirmDialog(splitPane, "Are you sure you want to delete the item: "
							+ selectedName + " and any child items?\n", "Confirm Deletion: " + selectedName,
							JOptionPane.OK_CANCEL_OPTION);
					if (s1 == 0) {
						DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree
								.getLastSelectedPathComponent();
						if (selectedNode != null) {
							TreePath tp = tree.getSelectionPath().getParentPath();
							model.removeNodeFromParent(selectedNode);
							// SQL execution
							Connection conn = null;
							try {
								Class.forName(driver).newInstance();
								conn = DriverManager.getConnection(url, user, pass);
								Statement stmt = conn.createStatement();

								// Delete all items which were under the selected item.
								updateStatement = "DELETE FROM items WHERE location LIKE '" + selectedLocation + "|"
										+ selectedName + "%' AND projectKey = " + projectKey + ";";
								if (logFlag)
									logMessage(3, updateStatement);
								updateStatus = stmt.executeUpdate(updateStatement);

								// Delete the selected item.
								updateStatement = "DELETE FROM items WHERE location = '" + selectedLocation
										+ "' AND name ='" + selectedName + "' AND projectKey = " + projectKey + ";";
								if (logFlag)
									logMessage(3, updateStatement);
								updateStatus = stmt.executeUpdate(updateStatement);
								conn.close();
							} catch (Exception e1) {
								if (logFlag)
									logMessage(2, "Error when deleting an item and all its children in Delete item.");
							}
							tree.setSelectionPath(tp);
						}
					}
				}
			}
		};
		/**
		 * The action listener for the nine buttons in the item panel(next, previous, add before, replace, add after,
		 * delete, plus(+), minus(-) and rotate right). The next and previous buttons are displayed when there is more
		 * than one image for an item. Adding, replacing and deleting images(add before, replace, add after and delete
		 * buttons) are the maintenance functions for item images. They are displayed when maintenance mode is turned on
		 * (the default) and there is at least one image for the item. If there are no images to display the replace
		 * button text is changed to "Add".
		 */
		ActionListener al = new ActionListener() {
			String updateStatement = new String();

			public void actionPerformed(ActionEvent e) {
				String command = e.getActionCommand();
				/**
				 * Next: displays the next image for the item. If this is the last image for the item the first image
				 * will be displayed. The item header information is changed to reflect the name of the new image and
				 * the image count.
				 */
				if (command.equals("next") && photoCount > 1) {
					zoomWidth = 0;
					zoomHeight = 0;
					rotateDegrees = 0;
					rotateMode = 0;
					/** If this is the last photo reset to the first */
					if (currentPhoto == photoCount - 1) {
						updateImage(installPath + projectName + "/" + itemPhotos[0]);
						selectedImage = itemPhotos[0];
						currentPhoto = 0;
					} else {
						updateImage(installPath + projectName + "/" + itemPhotos[currentPhoto + 1]);
						selectedImage = itemPhotos[currentPhoto + 1];
						currentPhoto++;
					}
					itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
							+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
							+ "\nDescription:  " + selectedDescription);
					itemInfoText.setCaretPosition(0);
					/**
					 * Previous: displays the previous image for the item. If this is the first image for the item the
					 * last image will be displayed. The item header information is changed to reflect the name of the
					 * new image and the image count.
					 */
				} else if (command.equals("previous") && photoCount > 1) {
					zoomWidth = 0;
					zoomHeight = 0;
					rotateDegrees = 0;
					rotateMode = 0;
					if (currentPhoto == 0) {
						updateImage(installPath + projectName + "/" + itemPhotos[photoCount - 1]);
						selectedImage = itemPhotos[photoCount - 1];
						currentPhoto = photoCount - 1;
					} else {
						updateImage(installPath + projectName + "/" + itemPhotos[currentPhoto - 1]);
						selectedImage = itemPhotos[currentPhoto - 1];
						currentPhoto = currentPhoto - 1;
					}
					itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
							+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
							+ "\nDescription:  " + selectedDescription);
					itemInfoText.setCaretPosition(0);
					/**
					 * Replace: Allows the user to select an image in the OS file system using an enhanced file chooser
					 * (chooseImage). If the user selects an image and presses the OK button the new image is copied to
					 * the project's image directory and the image table is updated with the new image name for the
					 * item. If there were no images (photoCount == 0) for the item a new record is inserted into the
					 * image table(Add... button). The new image is displayed and the item information is updated to
					 * show the new image name. Image buttons are checked for visibility due to the potential increase
					 * in the image count as a result of an "Add...".
					 */
				} else if (command.equals("replace")) {
					updateStatement = new String();
					replacedImage = selectedImage;
					selectedImage = chooseImage(splitPane);
					if (selectedImage.equals("No File Selected")) {
						selectedImage = replacedImage;
					} else {
						updateImage(installPath + projectName + "/" + selectedImage);
						if (photoCount == 0) {
							updateStatement = "INSERT INTO images (imageName, itemKey, imageOrder) VALUES('"
									+ selectedImage + "' , " + selectedKey + ", 1);";
							if (logFlag)
								logMessage(3, updateStatement);
							replaceButton.setText("Replace...");
							photoCount = 1;
							currentPhoto = 0;
							itemPhotos[currentPhoto] = selectedImage;
						} else {
							updateStatement = "UPDATE images SET imageName = '" + selectedImage + "' WHERE itemkey = '"
									+ selectedKey + "' AND imageName = '" + replacedImage + "';";
							if (logFlag)
								logMessage(3, updateStatement);
							itemPhotos[currentPhoto] = selectedImage;
						}
						// SQL execution
						Connection conn = null;
						try {
							Class.forName(driver).newInstance();
							conn = DriverManager.getConnection(url, user, pass);
							Statement stmt = conn.createStatement();
							updateStatus = stmt.executeUpdate(updateStatement);
							conn.close();
						} catch (Exception e1) {
							if (logFlag)
								logMessage(2,
										"Error when inserting or updating to image table when Add or Replace image.");
						}
						itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
								+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
								+ "\nDescription:  " + selectedDescription);
						itemInfoText.setCaretPosition(0);
						if (photoCount == 1) {
							nextButton.setVisible(false);
							previousButton.setVisible(false);
						} else {
							nextButton.setVisible(true);
							previousButton.setVisible(true);
						}
						replaceButton.setText("Replace...");
						if (maintenanceMode) {
							replaceButton.setVisible(true);
							addAfterButton.setVisible(true);
							addBeforeButton.setVisible(true);
							deleteButton.setVisible(true);
						} else {
							replaceButton.setVisible(false);
							addAfterButton.setVisible(false);
							addBeforeButton.setVisible(false);
							deleteButton.setVisible(false);
						}
						plusButton.setVisible(true);
						minusButton.setVisible(true);
						rotateRightButton.setVisible(true);
						zoomWidth = 0;
						zoomHeight = 0;
						rotateDegrees = 0;
						rotateMode = 0;
						updateImage(installPath + projectName + "/" + selectedImage);
					}
					/**
					 * Insert > (insert after): Allows the user to select an image in the OS file system using an
					 * enhanced file chooser (chooseImage). If the user selects an image and presses the OK button the
					 * new image is copied to the project's image directory. All the images for the item are then
					 * removed from the image table and reinserted in their new order. The new image is displayed and
					 * the item information is updated to show the new image name, order and image count. Visibility of
					 * buttons are checked due to the increase in the number of images.
					 */
				} else if (command.equals("addAfter")) {
					if (logFlag)
						logMessage(1, "Inserting image after current.");
					updateStatement = new String();
					replacedImage = selectedImage;
					selectedImage = chooseImage(splitPane);
					if (selectedImage == "No File Selected") {
						selectedImage = replacedImage;
					} else {
						updateImage(installPath + projectName + "/" + selectedImage);
						for (int i = photoCount - 1; i > currentPhoto; i--) {
							itemPhotos[i + 1] = itemPhotos[i];
						}
						photoCount++;
						currentPhoto++;
						itemPhotos[currentPhoto] = selectedImage;
						String updateStatement = "DELETE FROM images WHERE itemKey =  " + selectedKey;
						if (logFlag)
							logMessage(3, updateStatement);
						// SQL execution
						Connection conn = null;
						try {
							Class.forName(driver).newInstance();
							conn = DriverManager.getConnection(url, user, pass);
							Statement stmt = conn.createStatement();
							updateStatus = stmt.executeUpdate(updateStatement);
							for (int i = 0; i < photoCount; i++) {
								updateStatement = "INSERT INTO images (imageName,itemKey,imageOrder) VALUES('"
										+ itemPhotos[i] + "' , " + selectedKey + ", " + (i + 1) + ")";
								updateStatus = stmt.executeUpdate(updateStatement);
								if (logFlag)
									logMessage(3, updateStatement);
							}
							conn.close();
						} catch (Exception e1) {
							if (logFlag)
								logMessage(2,
										"Error when deleting and reinserting images into image table in Add After");
						}
					}
					itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
							+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
							+ "\nDescription:  " + selectedDescription);
					itemInfoText.setCaretPosition(0);
					if (photoCount == 1) {
						addAfterButton.setVisible(false);
						addBeforeButton.setVisible(false);
						deleteButton.setVisible(true);
					} else {
						if (maintenanceMode) {
							addAfterButton.setVisible(true);
							addBeforeButton.setVisible(true);
							deleteButton.setVisible(true);
						} else {
							addAfterButton.setVisible(false);
							addBeforeButton.setVisible(false);
							deleteButton.setVisible(false);
						}
					}
					nextButton.setVisible(true);
					previousButton.setVisible(true);
					plusButton.setVisible(true);
					minusButton.setVisible(true);
					rotateRightButton.setVisible(true);
					zoomWidth = 0;
					zoomHeight = 0;
					rotateDegrees = 0;
					rotateMode = 0;
					/**
					 * Insert < (insert before): Allows the user to select an image in the OS file system using an
					 * enhanced file chooser (chooseImage). If the user selects an image and presses the OK button the
					 * new image is copied to the project's image directory. All the images for the item are then
					 * removed from the image table and reinserted in their new order. The new image is displayed and
					 * the item information is updated to show the new image name, order and image count. Visibility of
					 * buttons are checked due to the increase in the number of images.
					 */
				} else if (command.equals("addBefore")) {
					if (logFlag)
						logMessage(1, "Inserting image before current.");
					updateStatement = new String();
					replacedImage = selectedImage;
					selectedImage = chooseImage(splitPane);
					if (selectedImage == "No File Selected") {
						selectedImage = replacedImage;
					} else {
						updateImage(installPath + projectName + "/" + selectedImage);
						for (int i = photoCount - 1; i >= currentPhoto; i--) {
							itemPhotos[i + 1] = itemPhotos[i];
						}
						itemPhotos[currentPhoto] = selectedImage;
						photoCount++;
						String updateStatement = "DELETE FROM images WHERE itemKey =  " + selectedKey;
						if (logFlag)
							logMessage(3, updateStatement);
						// SQL execution
						Connection conn = null;
						try {
							Class.forName(driver).newInstance();
							conn = DriverManager.getConnection(url, user, pass);
							Statement stmt = conn.createStatement();
							updateStatus = stmt.executeUpdate(updateStatement);
							for (int i = 0; i < photoCount; i++) {
								updateStatement = "INSERT INTO images (imageName,itemKey,imageOrder) VALUES('"
										+ itemPhotos[i] + "' , " + selectedKey + ", " + (i + 1) + ")";
								updateStatus = stmt.executeUpdate(updateStatement);
								if (logFlag)
									logMessage(3, updateStatement);
							}
							conn.close();
						} catch (Exception e1) {
							if (logFlag)
								logMessage(2,
										"Error when deleteing and reinserting images into image table in Add Before");
						}
					}
					itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
							+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
							+ "\nDescription:  " + selectedDescription);
					itemInfoText.setCaretPosition(0);
					if (photoCount == 1) {
						addAfterButton.setVisible(false);
						addBeforeButton.setVisible(false);
						deleteButton.setVisible(true);
					} else {
						if (maintenanceMode) {
							addAfterButton.setVisible(true);
							addBeforeButton.setVisible(true);
							deleteButton.setVisible(true);
						} else {
							addAfterButton.setVisible(false);
							addBeforeButton.setVisible(false);
							deleteButton.setVisible(false);
						}
					}
					nextButton.setVisible(true);
					previousButton.setVisible(true);
					plusButton.setVisible(true);
					minusButton.setVisible(true);
					rotateRightButton.setVisible(true);
					zoomWidth = 0;
					zoomHeight = 0;
					rotateDegrees = 0;
					rotateMode = 0;
					/**
					 * Delete: Allows the user to delete the selected image for an item. The user is first prompted to
					 * validate the deletion. Once the user confirms the request all the images for the item are then
					 * removed from the image table and reinserted in their new order. The previous image is displayed
					 * and the item information is updated to show the new image name, order and image count. Visibility
					 * of buttons are checked due to the decrease in the number of images.
					 */
				} else if (command.equals("delete")) {
					if (logFlag)
						logMessage(1, "Deleting current image.");
					int s1 = JOptionPane.showConfirmDialog(splitPane, "Are you sure you want to delete this image?\n",
							"Confirm Image Deletion", JOptionPane.OK_CANCEL_OPTION);
					if (s1 == 0) {
						updateStatement = new String();

						if (photoCount != 0) {
							if (photoCount == 1) {
								updateImage(installPath + "blank.jpg");
								photoCount = 0;
								currentPhoto = -1;
							} else {
								if (photoCount - 1 == currentPhoto) {
									updateImage(installPath + projectName + "/" + itemPhotos[0]);
									currentPhoto = 0;
									selectedImage = itemPhotos[currentPhoto];
								} else {
									updateImage(installPath + projectName + "/" + itemPhotos[currentPhoto + 1]);
									selectedImage = itemPhotos[currentPhoto + 1];
									for (int i = currentPhoto; i < photoCount - 1; i++) {
										itemPhotos[i] = itemPhotos[i + 1];
									}
								}
								photoCount--;
							}
							String updateStatement = "DELETE FROM images WHERE itemKey =  " + selectedKey + ";";
							if (logFlag)
								logMessage(3, updateStatement);
							// SQL execution
							Connection conn = null;
							try {
								Class.forName(driver).newInstance();
								conn = DriverManager.getConnection(url, user, pass);
								Statement stmt = conn.createStatement();
								updateStatus = stmt.executeUpdate(updateStatement);

								for (int i = 0; i < photoCount; i++) {
									updateStatement = "INSERT INTO images (imageName,itemKey,imageOrder) VALUES('"
											+ itemPhotos[i] + "' , " + selectedKey + ", " + (i + 1) + ")";
									updateStatus = stmt.executeUpdate(updateStatement);
									if (logFlag)
										logMessage(3, updateStatement);
								}

								conn.close();
							} catch (Exception e1) {
								if (logFlag)
									logMessage(2,
											"Error when deleting and reinserting images into image table in Delete image.");
							}
						}
						if (photoCount == 0) {
							itemInfoText.setText(selectedName + "                     ("
									+ "No Photos Added)   Item Type = " + selectedType + "\nDescription:  "
									+ selectedDescription);
						} else {
							itemInfoText.setText(selectedName + "                     (" + (currentPhoto + 1) + " of "
									+ photoCount + ")   " + selectedImage + "   Item Type = " + selectedType
									+ "\nDescription:  " + selectedDescription);

						}
						itemInfoText.setCaretPosition(0);
						if (photoCount == 0) {
							replaceButton.setText("Add...");
							if (maintenanceMode)
								replaceButton.setVisible(true);
							else
								replaceButton.setVisible(false);
							addAfterButton.setVisible(false);
							addBeforeButton.setVisible(false);
							deleteButton.setVisible(false);
							nextButton.setVisible(false);
							previousButton.setVisible(false);
							plusButton.setVisible(false);
							minusButton.setVisible(false);
							rotateRightButton.setVisible(false);
						} else if (photoCount == 1) {
							replaceButton.setText("Replace...");
							if (maintenanceMode) {
								replaceButton.setVisible(true);
								addAfterButton.setVisible(true);
								addBeforeButton.setVisible(true);
								deleteButton.setVisible(true);
							} else {
								replaceButton.setVisible(false);
								addAfterButton.setVisible(false);
								addBeforeButton.setVisible(false);
								deleteButton.setVisible(false);
							}
							nextButton.setVisible(false);
							previousButton.setVisible(false);
							plusButton.setVisible(true);
							minusButton.setVisible(true);
							rotateRightButton.setVisible(true);
						} else {
							if (maintenanceMode) {
								replaceButton.setVisible(true);
								addAfterButton.setVisible(true);
								addBeforeButton.setVisible(true);
								deleteButton.setVisible(true);
							} else {
								replaceButton.setVisible(false);
								addAfterButton.setVisible(false);
								addBeforeButton.setVisible(false);
								deleteButton.setVisible(false);
							}
							nextButton.setVisible(true);
							previousButton.setVisible(true);
							plusButton.setVisible(true);
							minusButton.setVisible(true);
							rotateRightButton.setVisible(true);
						}
						zoomWidth = 0;
						zoomHeight = 0;
						rotateDegrees = 0;
						rotateMode = 0;
					}
					/**
					 * When the user presses the zoom(+) or shrink(-) button their value is adjusted by the amount set
					 * here. The amount should be in a settings menu panel. The method updateImage uses these values
					 * when displaying the current image.
					 */
				} else if (command.equals("plus")) {
					zoomWidth = zoomWidth + 100;
					zoomHeight = zoomHeight + 100;
					updateImage(installPath + projectName + "/" + selectedImage);
				} else if (command.equals("minus")) {
					zoomWidth = zoomWidth - 100;
					zoomHeight = zoomHeight - 100;
					updateImage(installPath + projectName + "/" + selectedImage);
					/**
					 * When the user presses the rotate right button their value is adjusted by the amount set here. The
					 * amount should be in a settings menu panel. The method updateImage uses these values when
					 * displaying the current image.
					 */
				} else if (command.equals("rotateR")) {
					rotateMode = 1;
					rotateDegrees++;
					if (rotateDegrees > 3)
						rotateDegrees = 0;
					updateImage(installPath + projectName + "/" + selectedImage);
				}
			}
		};

		nextButton.addActionListener(al);
		previousButton.addActionListener(al);
		replaceButton.addActionListener(al);
		deleteButton.addActionListener(al);
		addAfterButton.addActionListener(al);
		addBeforeButton.addActionListener(al);
		plusButton.addActionListener(al);
		minusButton.addActionListener(al);
		rotateRightButton.addActionListener(al);

		nextButton.setActionCommand("next");
		previousButton.setActionCommand("previous");
		replaceButton.setActionCommand("replace");
		addAfterButton.setActionCommand("addAfter");
		addBeforeButton.setActionCommand("addBefore");
		deleteButton.setActionCommand("delete");
		plusButton.setActionCommand("plus");
		minusButton.setActionCommand("minus");
		rotateRightButton.setActionCommand("rotateR");

		addItemButton.addActionListener(alItem);
		editItemButton.addActionListener(alItem);
		deleteItemButton.addActionListener(alItem);

		addItemButton.setActionCommand("add");
		editItemButton.setActionCommand("edit");
		deleteItemButton.setActionCommand("delete");

		// Create a split pane with the two scroll panes in it.
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, treePanel, itemPanel);
		splitPane.setOneTouchExpandable(true);
		splitPane.setDividerLocation(230);
		splitPane.setPreferredSize(new Dimension(850, 450));
		splitPane.setSize(850, 450);
	}

	/**
	 * chooseImage: Displays the standard OS file chooser with an accessory allowing the user to see a thumb nail
	 * preview of the image before selection. A filter is applied requesting only jpg suffixes. The initial position is
	 * set to the users home directory but it remembers the users last position while running. If the file is selected
	 * it is copied to the project's image directory. The method returns either "No File Selected" or the name of the
	 * image file.
	 */
	public String chooseImage(Container container) {
		File sfile = null;
		File tfile = null;

		JFileChooser fc;
		fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		fc.setFileFilter(new SQLfilter());
		fc.setApproveButtonText("Select Image");
		fc.setDialogTitle("Select An Image for: " + selectedName);
		fc.setCurrentDirectory(new File(lastDirectory));
		ImagePreviewPanel preview = new ImagePreviewPanel();
		fc.setAccessory(preview);
		fc.addPropertyChangeListener(preview);

		if (fc.showOpenDialog(container) == JFileChooser.CANCEL_OPTION) {
			if (logFlag)
				logMessage(1, "No File Selected from chooseImage.");
			return ("No File Selected");
		} else {
			sfile = fc.getSelectedFile();
			String source = sfile.getPath();
			int sLength = sfile.getName().length();
			int pLength = sfile.getPath().length();
			source = source.substring(0, pLength - sLength - 1);
			source = source.replace("\\", "");
			String target = (installPath + projectName).replaceAll("/", "");
			if (!source.equals(target)) {
				tfile = new File(installPath + projectName + "/" + sfile.getName());
				try {
					copy(sfile, tfile);
				} catch (IOException e) {
					if (logFlag)
						logMessage(2, "IOException while trying to copy an image file: " + sfile.getName());
				}
			}
			lastDirectory = sfile.getPath().toString().replaceAll(sfile.getName(), "");
		}
		return sfile.getName();
	}

	/**
	 * If expand is true, expands all nodes in the tree. Otherwise, collapses all nodes in the tree.
	 */
	public void expandAll(JTree tree, boolean expand) {
		TreeNode root = (TreeNode) tree.getModel().getRoot();
		// Traverse tree from root
		expandAll(tree, new TreePath(root), expand);
	}

	private void expandAll(JTree tree, TreePath parent, boolean expand) {
		// Traverse children
		TreeNode node = (TreeNode) parent.getLastPathComponent();
		if (node.getChildCount() >= 0 && parent.getPathCount() < 3) {
			for (Enumeration<?> e = node.children(); e.hasMoreElements();) {
				TreeNode n = (TreeNode) e.nextElement();
				TreePath path = parent.pathByAddingChild(n);
				expandAll(tree, path, expand);
			}
		}
		// Expansion or collapse must be done bottom-up
		if (expand) {
			tree.expandPath(parent);
		} else {
			tree.collapsePath(parent);
		}
	}

	private static String getDateTime() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		Date date = new Date();
		return dateFormat.format(date);
	}

	public static void logMessage(int msgType, String output) {
		try {
			if (msgType == 1)
				msgWriter.write(getDateTime() + "  Message   = " + output + newline);
			else if (msgType == 2)
				msgWriter.write(getDateTime() + "  Error Msg = " + output + newline);
			else if (msgType == 3)
				msgWriter.write(getDateTime() + "  SQL Stmt  = " + output + newline);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Method: updateImage uses the name parameter to display a specific image. If the rotate button has been pressed
	 * then the image is rotated 90 degrees in a clockwise manner. The image is normally sized to the dimensions of the
	 * image panel unless the zoom (+) or shrink (-) buttons are pressed. If the user presses the zoom or shrink buttons
	 * the image is scaled to a predefined amount (about 10 percent).
	 * 
	 * @param name
	 *            - project location and name of the image to display.
	 */
	protected void updateImage(String name) {
		ImageIcon screenImage = createImageIcon(name);
		BufferedImage bi = null;
		BufferedImage biFlip = null;
		if (screenImage == null)
			return;
		/**
		 * Perform Rotation - if the user has requested that the image be rotated 90 degrees by pressing the rotate
		 * button.
		 */
		if (rotateMode == 1) {
			bi = new BufferedImage(screenImage.getImage().getWidth(null), screenImage.getImage().getHeight(null),
					BufferedImage.TYPE_INT_ARGB);
			// Copy image to buffered image
			Graphics g = bi.createGraphics();
			//
			// Paint the image onto the buffered image
			g.drawImage(screenImage.getImage(), 0, 0, null);

			int width = bi.getWidth();
			int height = bi.getHeight();
			/**
			 * The rotate right button sets the value for rotate degrees to 1 for 90 degrees, 2 for 180 degrees and 3
			 * for 270 degrees. This algorithm is a brute force approach where you just move every pixels from the
			 * current image(bi) to its proper place in the rotated image(biFlip) depending on the value of rotate
			 * degrees.
			 */
			if (rotateDegrees == 1) {
				biFlip = new BufferedImage(height, width, bi.getType());
				for (int i = 0; i < width; i++)
					for (int j = 0; j < height; j++)
						biFlip.setRGB(height - 1 - j, i, bi.getRGB(i, j));
			} else if (rotateDegrees == 2) {
				biFlip = new BufferedImage(width, height, bi.getType());
				for (int i = 0; i < height; i++)
					for (int j = 0; j < width; j++)
						biFlip.setRGB(j, height - 1 - i, bi.getRGB(j, i));
			} else if (rotateDegrees == 3) {
				biFlip = new BufferedImage(height, width, bi.getType());
				for (int i = 0; i < width; i++)
					for (int j = 0; j < height; j++)
						biFlip.setRGB(j, width - 1 - i, bi.getRGB(i, j));
			} else {
				biFlip = bi;
			}
//			bi = biFlip;
			screenImage = new ImageIcon(Toolkit.getDefaultToolkit().createImage(biFlip.getSource()));
		}
		/**
		 * Fit the selected image to the display window.
		 * 
		 * Using the image width and height and the current screen width and height this code will determine the best
		 * fit to display the entire image in the image panel.
		 * 
		 * Input: image and screen width and height. Output: new image width and height.
		 */
		float imageHeight = screenImage.getIconHeight();
		float imageWidth = screenImage.getIconWidth();

		float screenWidth = splitPane.getWidth() - splitPane.getDividerLocation() - 10;
		float screenHeight = splitPane.getHeight() - 110;

		float newHeight = 0;
		float newWidth = 0;

		if (imageHeight < screenHeight && imageWidth > screenWidth) {
			newWidth = screenWidth;
			newHeight = (screenWidth * imageHeight) / imageWidth;
		} else if (imageHeight > screenHeight && imageWidth < screenWidth) {
			newHeight = screenHeight;
			newWidth = imageWidth * (screenHeight / imageHeight);
		} else {
			newWidth = screenWidth;
			newHeight = (screenWidth * imageHeight) / imageWidth;
			if (newHeight > screenHeight) {
				newHeight = screenHeight;
				newWidth = (screenHeight * imageWidth) / imageHeight;
			}
		}

		Image img = screenImage.getImage();
		/**
		 * This code scales the image to the fit the current screen size. If the user is using the zoom in or out
		 * buttons the size is adjusted accordingly.
		 */
		Image img2 = img.getScaledInstance((int) newWidth + zoomWidth, (int) newHeight + zoomHeight, 0);
		ImageIcon screenImage2 = new ImageIcon(img2);
		/** Sets the image into the image panel */
		picture.setIcon(screenImage2);
		if (picture != null) {
			picture.setSize((int) newWidth, (int) newHeight);
		} else {
			picture.setText("Image not found");
		}
	}

	/** Used by TreasureBox */
	public JTree getTree() {
		return tree;
	}

	/** Used by TreasureBox */
	public JSplitPane getSplitPane() {
		return splitPane;
	}

	/** Returns an ImageIcon, or null if the path was invalid. */
	protected ImageIcon createImageIcon(String path) {
		ImageIcon image = new ImageIcon();
		String URLPath = new String();
		java.net.URL imgURL = Finder.class.getResource(path);
		if (imgURL != null) {
			image = new ImageIcon(imgURL);
			return image;
		}
		URLPath = path.replace("\\", "/");
		URLPath = "file:///" + URLPath;
		try {
			imgURL = new java.net.URL(URLPath);
		} catch (MalformedURLException e) {
			if (logFlag)
				logMessage(2, "Couldn't find file: " + URLPath + " " + path);
			return null;
		}
		image = new ImageIcon(imgURL);
		return image;
	}

	/**
	 * Method: createNodes Parameter: projectPrompt Prompts the user with the project management panel.
	 * 
	 * Description: This method builds a node tree for a given project. It uses the current projectKey to retrieve all
	 * the records for a project from the database. It is called from three places: 1) when the program starts
	 * (projectPrompt = true),2) in the menu item "Manage Projects" (projectPrompt = true) and 3) in the menu item
	 * "Reorder Items" (projectPrompt = false)
	 */
	public int createNodes(boolean projectPrompt) {
		DefaultMutableTreeNode child = null;
		DefaultMutableTreeNode parent = null;
		int beginIndex = 0;
		int endIndex = -1;
		int cc = 0;
		int ans = 0;

		String[] loc = new String[100];
		if (projectPrompt) {
			/** Prompt the user for a project */
			ProjectDialog pd = new ProjectDialog(frame, true, projectKey);
			ans = pd.getAnswer();
			/**
			 * If the user selected an existing project(ans = 1) or the user added a new project(ans = 2) get the
			 * project key, name and description from the project dialog and turn on maintenance mode since the user
			 * will need to enter data.
			 */
			if (ans > 0) {
				projectKey = pd.getProjectKey();
				projectName = pd.getName();
				projectDescription = pd.getDescription();
				maintenanceMode = true;
			} else {
				/**
				 * If the user deleted the current project and then canceled the dialog, exit the program.
				 */
				if (projectKey == null || pd.getKeyChange())
					System.exit(1);
			}
		}
		/**
		 * If the user was prompted for a project and then pressed the cancel button, return a code of 1 to the caller
		 * indicating no change was made.
		 */
		if (projectPrompt && ans == 0) {
			return 1;
		} else {
			/**
			 * Read the all the items for a given project and build the node structure to prepare the item tree for
			 * display.
			 */
			Connection conn = null;
			try {
				Class.forName(driver).newInstance();
				conn = DriverManager.getConnection(url, user, pass);
				Statement stmt = conn.createStatement();
				ResultSet rs;

				String query = "SELECT name, itemOrder, location, type, level, container FROM items WHERE projectKey = "
						+ projectKey + " ORDER BY level, location, itemOrder;";
				if (logFlag)
					logMessage(3, query);
				rs = stmt.executeQuery(query);
				while (rs.next()) {
					/**
					 * Add each database record to the tree. Since the returned records are sorted by level the tree can
					 * be built from the top down.
					 */
					String Name = rs.getString("name");
					String Type = rs.getString("type");
					String Location = rs.getString("location");
					int Level = rs.getInt("level");
					String Container = rs.getString("container");

					/**
					 * The returned location is parsed into individual tree node names. The names are the values of the
					 * tree path where the item is located. These names are put into the loc[] array.
					 */
					for (int i = 0; i < Level; i++) {
						beginIndex = endIndex + 1;
						endIndex = Location.indexOf("|", beginIndex);
						if (endIndex == -1)
							loc[i] = Location.substring(beginIndex);
						else
							loc[i] = Location.substring(beginIndex, endIndex);
					}
					/**
					 * If this is the first node (i.e. the root of the tree) add it as top of the tree.
					 */
					if (Type.equals("ROOT")) {
						treeTop = new DefaultMutableTreeNode(Name);
						parent = treeTop;
						rootName = Name;
						/**
						 * If the row belongs to the root just insert it under the root.
						 */
					} else if (Location.equals(rootName)) {
						if (Container.equals("Y"))
							child = new DefaultMutableTreeNode(Name, true);
						else
							child = new DefaultMutableTreeNode(Name, false);
						model.insertNodeInto(child, parent, parent.getChildCount());
						/** Otherwise use the Location to find the parent */
					} else {
						/**
						 * Match the nodes in the Location variable with the tree's nodes. When the tree path matches
						 * the database record location the item name is placed under the last node specified in the
						 * tree path.
						 */
						parent = treeTop;

						for (int i = 0; i < Level; i++) {
							cc = parent.getChildCount();
							if (cc == 1) {
								if (loc[i].equals(parent.getChildAt(0).toString())) {
									parent = (DefaultMutableTreeNode) parent.getChildAt(0);
									continue;
								}
							} else {
								for (int j = 0; j < cc; j++) {
									if (parent.getChildAt(j).toString().equals(loc[i])) {
										parent = (DefaultMutableTreeNode) parent.getChildAt(j);
										break;
									}
								}
							}
						}

						/**
						 * Set the user specified value for the node 1. can have children (true) 2. no children (simple
						 * object) (false)
						 */
						if (Container.equals("Y")) {
							child = new DefaultMutableTreeNode(Name, true);
						} else {
							child = new DefaultMutableTreeNode(Name, false);
						}

						/**
						 * Add the new item under the last node specified in the Location
						 */
						model.insertNodeInto(child, parent, parent.getChildCount());
					}
				}
				conn.close();
			} catch (Exception e1) {
				if (logFlag)
					logMessage(2, "Error when building the node structure for a project in createNodes()");
			}
			return 0;
		}
	}

	/**
	 * Fast & simple file copy. This method is called when the user selects an image using the file chooser. Once
	 * selected the image is transfered to the projects image folder
	 */
	public static void copy(File source, File dest) throws IOException {
		FileChannel in = null, out = null;
		try {
			in = new FileInputStream(source).getChannel();
			out = new FileOutputStream(dest).getChannel();
			long size = in.size();
			MappedByteBuffer buf = in.map(FileChannel.MapMode.READ_ONLY, 0, size);
			out.write(buf);
		} finally {
			if (in != null)
				in.close();
			if (out != null)
				out.close();
		}
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be invoked from the event-dispatching thread.
	 */
	private static void createAndShowGUI() {
		JFrame frame = new JFrame("Finder");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Create and set up the window.
		Finder finder = new Finder();
		frame.getContentPane().add(finder.getSplitPane());

		// Display the window.
		frame.pack();
		frame.setVisible(true);

	}

	/**
	 * Schedule a job for the event-dispatching thread: creating and showing this application's GUI.
	 */
	public static void main(String[] args) {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}
}
