package tbox;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.Border;

public class HelpDialog extends JDialog {
	/**
	 * The HelpDialog displays the help information for the TreasureBox application.
	 */
	private static final long serialVersionUID = 1L;
	private JTextArea description = new JTextArea(40, 55);
	JPanel helpPanel = new JPanel(new FlowLayout());
	JButton closeButton;

	HelpDialog(JFrame frame) {
		super(frame, "Treasure Box Item Organizer Help", false);

		this.setMinimumSize(new Dimension(400, 225));
		this.setMaximumSize(new Dimension(400, 225));
		this.setResizable(true);
		description
				.setText("Treasure Box Help\n\n\n"
						+ "Overview:\n\n" 
						+ "         Treasure Box is a generalized item organizer where a tree structure is suitable for the data being\n"
						+ "         stored. Although TreasureBox can maintain several projects in its database only one project (tree)\n"
						+ "         is displayed per open window. The ROOT item in the tree is always the project name and its description\n"
						+ "         is the project description. The user adds items to the project tree arranging the data in a\n"
						+ "         hierarchical structure. Each item in the tree has a name, type, description and optionally one or more\n"
						+ "         images. If the item is set up as a Container other items can then be added under this item.\n\n\n"
						+ "Main Panel Components:\n\n"
						+ "  Location Panel - Keeps track of where you are in the item tree by concatenating the tree node names\n"
						+ "         together using a bar(|) as a separator character. All locations for items start with the project\n"
						+ "         name except for the ROOT item which has no location.\n\n"
						+ "  Search Panel - Allows the user to search the database for items which match the user's text and type\n"
						+ "         selection. The user can search the name, the description, or both. The search utility uses the\n"
						+ "         selected position in the item tree as the starting point of a search. Only when you select the\n"
						+ "         ROOT of the tree will the search look through all the items in a project. If you want to search\n"
						+ "         under a particular item just select that item before you press the search button.\n\n"
						+ "  Item Tree Panel - Allows the user to select an item from the tree for display. In maintenance mode the\n"
						+ "         user can create new items by selecting the parent item and pressing the add key. If the item was\n"
						+ "         not defined as a Container the Add Item button would not be displayed. The user has the ability\n"
						+ "         to edit or delete the selected item as well.\n\n"
						+ "  Item Display Panel - displays item information including name, description, type, current image and total\n"
						+ "         number of images for the item. When there are more than one image for an item the Next and Previous\n"
						+ "         buttons are displayed. Any image can be zoomed in with the plus(+) button or zoomed out with the\n"
						+ "         minus(-) button or rotated 90 degrees with the rotate right button. In maintenance mode new images\n"
						+ "         can be added, replaced or deleted. When new images are added they can be place before(<) or after(>)\n"
						+ "         the current image. Currently only jpg images are handled.\n\n"
						+ "  List Display Panel - When the user selects an item in the item tree panel the list display will contain any\n"
						+ "         child items directly under the selected item. If a search is requested all items which meet the search\n"
						+ "         are placed in the List display. Selecting an item in the List display is the same as selecting the same\n"
						+ "         item in the item tree.\n\n\n"
						+ "Dialog Components:\n\n"
						+ "  Project Dialog - When the TresureBox application starts the Project Dialog is displayed. This Dialog\n"
						+ "         provides the user with selection and maintenance functions for projects created in the database.\n"
						+ "         The user can select a project from the current list of projects in the database or create a new project\n"
						+ "         with a new name and description. A new project will create a ROOT item in the Tree Panel so other items\n"
						+ "         can be added to it. Projects can also be deleted from this Dialog. This will remove all items and images\n"
						+ "         from the database and remove all the images in the project directory as well as the directory itself.\n"
						+ "         The project name and description can be changed in the Main Panel (above) by selecting the ROOT item\n"
						+ "         in the Tree Panel and using the Edit Panel.\n"
						+ "  Add Dialog - When the Add item button is pressed the new item will be added directly under the currently\n"
						+ "         selected item. The dialog prompts the user for a name, description, type and whether or not it\n"
						+ "         is a Container (i.e. it can have other items added to it). If all the fields are validated (see the\n"
						+ "         field definitions below) the new item is added to the database.\n\n"
						+ "  Edit Dialog - When the Edit item button is pressed the selected item will be displayed in the Edit panel.\n"
						+ "         The user may change the item name, description, type and whether or not it is a Container\n"
						+ "         (i.e. it can have other items added to it). If all the fields are validated (see the field definitions\n"
						+ "         below) the item is modified in the database. Changing an item from 'Container' to 'Not a Container'\n"
						+ "         will delete any child items that were added to this item. Changing the name a ROOT (project) item\n"
						+ "         will correspondingly rename the directory where the project images reside.\n\n"
						+ "  Reorder Dialog - Allows the user to rearrange the order of the child items under the selected item(parent).\n"
						+ "         Items are normally arranged in the order that they are entered. If the user wishes to change the order\n"
						+ "         they must select the parent item and selecting the Item - Reorder (menu item) the Reorder Dialog will\n"
						+ "         allowing the user to adjust the order of the child items using the up and down arrows. When the user is\n"
						+ "         satisfied the item order will be updated in the Tree Panel and in the database for future use.\n\n\n"
						+ "Fields:\n\n"
						+ "  Item Name - is used as the node name for elements in the item tree. It is also used in the\n"
						+ "         Location field to help describe where an item is placed. The field is between one and\n"
						+ "         25 characters long inclusive. It must only contain alpha-numeric (A-Z, a-z, 0-9),\n"
						+ "         whitespace (spaces) or one of the seven special characters (-#@$%.:).\n\n"
						+ "  Item Type - is a user defined field for classifying items. For example a Home project may contain\n"
						+ "         types like ART, FUNITURE, BOOK, CD, DVD or TOOL. A Car project may contain types like\n"
						+ "         GM, Ford, or Fiat. The field is between one and 15 characters long inclusive. It must only\n"
						+ "         contain alpha-numeric (A-Z, a-z, 0-9), whitespace (spaces) or one of the seven special\n"
						+ "         characters (-#@$%.:).\n\n"
						+ "  Description - is a large text field that describes the item. The field is between one and\n"
						+ "         1000 characters long inclusive. There are no restrictions on the characters used.\n\n"
						+ "  Container - a switch set to either YES or NO indicating whether this item can contain other\n"
						+ "         items. When set to NO the Add Item button does not show up when the item is selected.\n\n\n"
						+ "Menu Commands\n\n"
						+ "  File - Manage Projects - Creates a project dialog which allows the user to select and manage\n"
						+ "         projects. From the dialog you can create a new project or select an existing project from\n"
						+ "         a list. A project can also be deleted from the database.\n\n"
						+ "  File - Toggle Maintenance Mode - This switch is used to control the buttons which are used for\n"
						+ "         maintenance of items and images. When you are building the project you need to add, edit\n"
						+ "         and delete items and images. Normally when you create a new project you want to be in\n"
						+ "         maintenance mode. Once the database is built and you are just browsing or searching you\n"
						+ "         may want to turn off maintenance mode and hide the maintenance buttons.\n\n"
						+ "  File - Windows Look and Feel - Sets up the display to a Microsoft Windows style for the graphical\n"
						+ "         components. This is the default setting.\n\n"
						+ "  File - Java Look and Feel - Sets up the display to a Java style for the graphical components.\n\n"
						+ "  File - Motif Look and Feel - Sets up the display to a Motif style for the graphical components.\n\n"
						+ "  File - Exit = Shuts down the TreasureBox application. There is nothing to save since all database\n"
						+ "         changes occurred when the user made a valid request.\n\n\n"
						+ "  Item - Full Screen Image - Resizes the currently selected image to fit the full display device.\n\n"
						+ "  Item - Reorder - When there is more than one item under a selected item they can be reordered\n"
						+ "         in any sequence that the user wishes. The default order is the order that the item was\n"
						+ "         entered. Once the change is made the display will reflect the changes and the database\n"
						+ "         will be updated to indicate the new order.\n\n\n"
						+ "  Tools - Refresh Screen - After the user re-sizes the screen (either bigger or smaller) or a new type\n"
						+ "         has been added the screen refresh adjusts the components to fit the new screen size and refreshes\n"
						+ "         the types in the search panel from the database\n\n"
						+ "  Tools - Print Current Image - print the currently displayed image on the printer. It is re-sized to fit\n"
						+ "         the output print size.\n\n"
						+ "  Tools - Print Item Report - prints the data for a single item including thumbnails of its images.\n\n"
						+ "  Tools - Turn on Trace - creates a file (treasureboxLog.txt) in the install directory. There are three\n"
						+ "          types of messages that are sent to the file: Informational Messages, Error Messages and SQL\n"
						+ "          Statements.\n\n\n"
						+ "  Tools - Turn off Trace - saves the file (treasureboxLog.txt) in the install directory so that the \n"
						+ "          user is able to view the log file.\n\n\n"
						+ "  Help - TreasureBox Help - this panel is displayed to the user\n\n");
		Border blackline;
		blackline = BorderFactory.createLineBorder(Color.black);
		description.setFont(new Font("TimesNewRoman", Font.PLAIN, 12));
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setEditable(false);
		description.setBorder(blackline);
		JScrollPane descriptionScrollPane = new JScrollPane(description);
		description.setCaretPosition(0);
		helpPanel.add(descriptionScrollPane);
		closeButton = new JButton("Close");
		ActionListener al = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (closeButton == e.getSource()) {
					setVisible(false);
				}
			}
		};
		closeButton.addActionListener(al);
		helpPanel.add(closeButton);
		getContentPane().add(helpPanel);
		pack();
		setVisible(true);
	}
}