package tbox;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class ProjectDialog extends JDialog {
	/**
	 * ProjectDialog provides a interface which allows the user to select or delete an existing project from the current
	 * project list. The user can optionally create a new project. The class returns the project key, name and
	 * description.
	 * 
	 * The "Add A New Project" button uses the entered name and description to insert a new project record into the
	 * project table of the Treasure Box database. The generated project key is subsequently retrieved to return it with
	 * the name and description to the calling application.
	 * 
	 * Selecting an element from the project list enables the delete project and select project buttons.
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Panels used to place graphical components onto the display
	 */
	JPanel control = new JPanel();
	JPanel selectedPanel = new JPanel();
	JPanel listPanel = new JPanel();
	JPanel addPanel = new JPanel();
	JPanel spacePanel = new JPanel();
	JPanel cancelPanel = new JPanel();
	JPanel title1Panel = new JPanel();
	JPanel title2Panel = new JPanel();

	JList displayList = new JList();
	JScrollPane listView = new JScrollPane(displayList);
	ListSelectionListener listSelectionListener = null;

	private JTextField selectedName = new JTextField(25);

	private JTextArea selectedDescription = new JTextArea(3, 40);

	private JButton addButton = null;
	private JButton selectButton = null;
	private JButton deleteButton = null;
	private JButton cancelButton = null;
	private int answer = 0;

	String projectKey = new String();
	String name = new String();
	String description = new String();

	String[] projects = new String[100];
	String[] p = new String[100];
	String[] n = new String[100];
	String[] d = new String[100];

	String updateStatement = new String();
	int updateStatus = 0;

	int pCnt = 0;
	int selection = 0;
	boolean pKeyDeleted = false;
	/**
	 * Set up the mySQL database access to the treasurebox database.
	 */
	final String url = "jdbc:mysql://localhost:3306/treasurebox";
	final String driver = "com.mysql.jdbc.Driver";
	final String user = "treasureboxuser";
	final String pass = "treasure";

	/**
	 * Determine the install path where the jar file resides. The command returns the path and name of the executable
	 * jar file. The name is removed to set the install path alone.
	 */
	StringBuffer OSPath = new StringBuffer(ProjectDialog.class.getProtectionDomain().getCodeSource().getLocation()
			.getPath());
	int pathEnd = OSPath.lastIndexOf("/");
	String installPath = OSPath.substring(0, pathEnd + 1);

	public String getProjectKey() {
		return (projectKey);
	}

	public String getName() {
		return (name);
	}

	public String getDescription() {
		return (description);
	}

	public int getAnswer() {
		return answer;
	}

	public boolean getKeyChange() {
		return pKeyDeleted;
	}

	public ProjectDialog(JFrame frame, boolean modal, final String pKey) {
		super(frame, "Treasure Box - Item Organizer", modal);
		// Use a black line border
		frame.setIconImage(Toolkit.getDefaultToolkit().getImage(installPath + "treasurebox.png"));
		Border blackline;
		blackline = BorderFactory.createLineBorder(Color.black);
		this.setMaximumSize(new Dimension(400, 325));
		this.setResizable(false);
		selectedPanel.setBackground(Color.YELLOW);
		addPanel.setBackground(Color.YELLOW);
		title1Panel.setBackground(Color.YELLOW);
		control.setBackground(Color.CYAN.darker());
		listPanel.setBackground(Color.CYAN.darker());
		title2Panel.setBackground(Color.CYAN.darker());
		spacePanel.setBackground(Color.BLUE);
		cancelPanel.setBackground(Color.BLUE);

		/**
		 * Get the list of projects from the database and place them into a String array to create the displayList.
		 */
		Connection conn = null;
		try {
			Class.forName(driver).newInstance();
			conn = DriverManager.getConnection(url, user, pass);
			Statement stmt = conn.createStatement();
			ResultSet rs;
			int addblanks = 0;

			String query = "SELECT projectKey, name, description FROM projects;";
			rs = stmt.executeQuery(query);
			while (rs.next()) {
				projectKey = rs.getString("projectKey");
				p[pCnt] = projectKey;
				name = rs.getString("name");
				n[pCnt] = name;
				addblanks = 20 - name.length();
				StringBuffer nameSB = new StringBuffer(name);
				for (int i = 0; i < addblanks; i++)
					nameSB.append(" ");
				name = new String(nameSB);
				description = rs.getString("description");
				d[pCnt] = description;
				if (description.length() > 100)
					projects[pCnt++] = name + description.substring(0, 100);
				else
					projects[pCnt++] = name + description;
			}
			conn.close();
		} catch (Exception e1) {
			System.err.println("Got an exception! ");
			System.err.println(e1.getMessage());
		}

		/** Handle the action events for the panel */
		ActionListener al = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/**
				 * Adds a new project to the database.
				 */
				if (addButton == e.getSource()) {
					if ((selectedName.getText().length() > 0) && (selectedDescription.getText().length() > 0)) {
						name = selectedName.getText();
						description = selectedDescription.getText();

						/** Create an image directory for the project */
						try {
							boolean success = (new File(installPath + name)).mkdir();
							if (success) {
								System.out.println("Directory: " + name + " created");
							}
						} catch (Exception e1) {// Catch exception if any
							System.err.println("Error: Could not create directory " + name + (e1).getMessage());
						}
						/** Insert a new record for the project */
						String updateStatement = "INSERT INTO projects (name, description) VALUES('" + name + "', '"
								+ selectedDescription.getText() + "');";
						System.out.println(updateStatement);
						Connection conn = null;
						try {
							Class.forName(driver).newInstance();
							conn = DriverManager.getConnection(url, user, pass);
							Statement stmt = conn.createStatement();
							stmt.executeUpdate(updateStatement);
							/**
							 * Retrieve the generated projectKey for the project
							 */
							String query = "SELECT projectKey FROM projects WHERE name = '" + selectedName.getText()
									+ "';";
							ResultSet rSet = stmt.executeQuery(query);
							while (rSet.next()) {
								projectKey = rSet.getString("projectKey");
							}
						} catch (Exception e1) {
							System.err.println("Got an exception! Selecting project key from projects;");
							System.err.println(e1.getMessage());
						}
						/**
						 * Insert the root record for the project into the Item table
						 */
						updateStatement = "INSERT INTO items (name, projectKey,itemOrder, description, container, type, location, locationsize, level)"
								+ " VALUES('"
								+ selectedName.getText()
								+ "', "
								+ projectKey
								+ ", 0, '"
								+ selectedDescription.getText() + "', 'Y', 'ROOT', ' ', 0, 0);";
						System.out.println(updateStatement);
						try {
							Statement stmt = conn.createStatement();
							updateStatus = stmt.executeUpdate(updateStatement);
							conn.close();
						} catch (Exception e1) {
							System.err.println("Got an exception! Inserting root record into items;");
							System.err.println(e1.getMessage());
						}
						answer = 1;
						setVisible(false);
						/**
						 * If the name and description are not valid display a message to the user.
						 */
					} else {
						JOptionPane.showMessageDialog(null, "A valid Project Name and Description are required.",
								"Cannot Add Project", JOptionPane.ERROR_MESSAGE);
					}
					/**
					 * Select Button - Return with the selected project from the list
					 */
				} else if (selectButton == e.getSource()) {
					answer = 2;
					setVisible(false);
					/**
					 * Cancel Button - Return with no changes
					 */
				} else if (cancelButton == e.getSource()) {
					answer = 0;
					setVisible(false);
					/**
					 * Delete Button - Return with the selected project from the list
					 */
				} else if (deleteButton == e.getSource()) {
					int s1 = JOptionPane.showConfirmDialog(null, "Are you sure you want to delete the Project: " + name
							+ "?\n", "Confirm Deletion: " + name, JOptionPane.OK_CANCEL_OPTION);
					if (s1 == 0) {
						for (int i = selection; i < pCnt; i++) {
							projects[i] = projects[i + 1];
							p[i] = p[i + 1];
							n[i] = n[i + 1];
							d[i] = d[i + 1];
						}
						pCnt--;
						displayList.repaint();
						/** Delete the image directory for the project */
						try {
							File[] listOfImages = (new File(installPath + name)).listFiles();
							int numberOfImages = listOfImages.length;
							for (int i = 0; i < numberOfImages; i++) {
								listOfImages[i].delete();
							}
							(new File(installPath + name)).delete();
						} catch (Exception e1) {// Catch exception if any
							System.err
									.println("Error: Could not delete files in directory " + name + (e1).getMessage());
						}
						/**
						 * Delete the selected project. This performs a cascaded delete on both the items and images
						 * tables.
						 */
						Connection conn1 = null;
						try {
							Class.forName(driver).newInstance();
							conn1 = DriverManager.getConnection(url, user, pass);
							Statement stmt1 = conn1.createStatement();
							updateStatement = "DELETE FROM projects WHERE projectKey = " + projectKey + ";";
							System.out.println(updateStatement);
							updateStatus = stmt1.executeUpdate(updateStatement);
							conn1.close();
						} catch (Exception e1) {
							System.err.println("Got an exception! Problem Deleting project");
							System.err.println(e1.getMessage());
						}

						displayList.clearSelection();
						selectButton.setEnabled(false);
						deleteButton.setEnabled(false);
						/**
						 * if the deleted project key was the current project notify the caller.
						 */
						if (projectKey.equals(pKey)) {
							pKeyDeleted = true;
						}
						projectKey = null;
						name = null;
						description = null;
					}
				}
			}
		};
		/**
		 * Once the user selects a project from the list the project key, name and description are set and the select
		 * and delete buttons are enabled.
		 */
		listSelectionListener = new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent listSelectionEvent) {
				JList list = (JList) listSelectionEvent.getSource();
				selection = list.getSelectedIndex();
				if (selection != -1 && selection < pCnt) {
					projectKey = p[selection];
					name = n[selection];
					description = d[selection];
					selectButton.setEnabled(true);
					deleteButton.setEnabled(true);
				}
			}
		};
		/**
		 * If the user double clicks in the project list, set the project key, name and description and close the
		 * dialog.
		 */
		MouseListener mouseListener = new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					int index = displayList.locationToIndex(e.getPoint());
					if (index != -1 && index < pCnt) {
						projectKey = p[index];
						name = n[index];
						description = d[index];
						answer = 2;
						setVisible(false);
					}
				}
			}
		};
		addButton = new JButton("Add New Project");
		addButton.addActionListener(al);
		addButton.setActionCommand("add");

		selectButton = new JButton("Select Project");
		selectButton.addActionListener(al);
		selectButton.setEnabled(false);
		selectButton.setActionCommand("select");

		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(al);
		cancelButton.setEnabled(true);
		cancelButton.setActionCommand("cancel");

		deleteButton = new JButton("Delete Project");
		deleteButton.addActionListener(al);
		deleteButton.setEnabled(false);
		deleteButton.setActionCommand("delete");

		displayList = new JList(projects);
		displayList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		displayList.addListSelectionListener(listSelectionListener);
		displayList.addMouseListener(mouseListener);

		displayList.setSize(250, 40);
		displayList.setFont(new Font("Courier", Font.BOLD, 12));
		listView = new JScrollPane(displayList);

		selectedPanel.setLayout(new BoxLayout(selectedPanel, BoxLayout.Y_AXIS));
		selectedPanel.setBorder(blackline);
		selectedPanel.add(spacePanel);
		selectedPanel.add(spacePanel);
		selectedPanel.add(spacePanel);
		title1Panel.setLayout(new BorderLayout());
		title1Panel.add(new JLabel("Or Start A New Project"), BorderLayout.WEST);
		selectedPanel.add(title1Panel);

		selectedPanel.add(new JLabel("Project Name"));
		selectedPanel.add(selectedName);
		selectedPanel.add(new JLabel("Project Description"));
		selectedDescription.setFont(new Font("Courier", Font.PLAIN, 12));
		selectedDescription.setLineWrap(true);
		selectedDescription.setWrapStyleWord(true);
		selectedDescription.setBorder(blackline);
		JScrollPane descriptionScrollPane = new JScrollPane(selectedDescription);

		selectedPanel.add(descriptionScrollPane);
		addPanel.setLayout(new BorderLayout());
		addPanel.add(addButton, BorderLayout.WEST);
		selectedPanel.add(addPanel);
		selectedPanel.add(spacePanel);
		cancelPanel.setLayout(new BorderLayout());
		cancelPanel.add(cancelButton, BorderLayout.EAST);
		selectedPanel.add(cancelPanel);

		spacePanel.add(new JLabel(" "));

		listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.Y_AXIS));
		listPanel.setBorder(blackline);
		title2Panel.setLayout(new BorderLayout());
		title2Panel.add(new JLabel("Select A Project:"), BorderLayout.WEST);
		listPanel.add(title2Panel);
		listPanel.add(new JLabel(" "));
		listPanel.add(listView);

		control.setLayout(new BorderLayout());
		control.add(new JLabel(" "), BorderLayout.SOUTH);
		control.add(selectButton, BorderLayout.WEST);
		control.add(deleteButton, BorderLayout.EAST);

		setLayout(new BorderLayout());
		add(listPanel, BorderLayout.NORTH);
		add(control, BorderLayout.CENTER);
		add(selectedPanel, BorderLayout.SOUTH);

		pack();
		setLocationRelativeTo(frame);
		setVisible(true);
	}
}
