package tbox;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.AffineTransform;
import java.awt.image.RenderedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;

import javax.imageio.ImageIO;
import javax.print.Doc;
import javax.print.DocFlavor;
import javax.print.DocPrintJob;
import javax.print.PrintService;
import javax.print.PrintServiceLookup;
import javax.print.ServiceUI;
import javax.print.SimpleDoc;
import javax.print.attribute.DocAttributeSet;
import javax.print.attribute.HashDocAttributeSet;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.event.PrintJobAdapter;
import javax.print.event.PrintJobEvent;
import javax.print.event.PrintJobListener;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.ResultSet;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;

public class TreasureBox extends JFrame implements Printable, FocusListener, TreeSelectionListener {
	/**
	 * The TreasureBox - Item Organizer application.
	 */
	private static final long serialVersionUID = 1L;
	public static JFrame frame = new JFrame();
	private JList displayList;
	JMenuBar menuBar = new JMenuBar();
	JScrollPane listView = new JScrollPane();
	JTextArea displayLocation = new JTextArea();
	JScrollPane locationView = new JScrollPane(displayLocation);
	JSplitPane top = new JSplitPane();
	JPanel search = new JPanel(new BorderLayout());
	JPanel searchBox = new JPanel(new BorderLayout());
	JTextField searchText = new JTextField("Enter Search Text Here: ", 20);
	String loc = new String();
	ListSelectionModel listSelectionModel;
	ListSelectionListener listSelectionListener = null;
	String[] displayListName = new String[300];
	String[] displayListLocation = new String[300];
	JComboBox types = new JComboBox();

	static String fileSeperator = System.getProperty("file.separator");
	static String newline = System.getProperty("line.separator");
	static String userDir = System.getProperty("user.home");
	/**
	 * Determine the install path where the jar file resides.
	 */
	static StringBuffer OSPath = new StringBuffer(ProjectDialog.class.getProtectionDomain().getCodeSource()
			.getLocation().getPath());
	static int pathEnd = OSPath.lastIndexOf("/");
	static String installPath = OSPath.substring(0, pathEnd + 1);

	private static String getDateTime() {
		DateFormat dateFormat = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss");
		Date date = new Date();
		return dateFormat.format(date);
	}

	/**
	 * This is where we add a finder split pane. It is a horizontal split pane and contains the tree and item image
	 * panels. Finder is in turn used as the top component in another vertical split pane where the selection list is on
	 * the bottom.
	 */
	static Finder finder = new Finder();
	/**
	 * The project key is obtained from the finder component since it handles project management.
	 */
	static String projectKey = finder.projectKey;

	JRadioButton nameButton = new JRadioButton();
	JRadioButton descButton = new JRadioButton();
	JRadioButton bothButton = new JRadioButton();

	JButton searchButton = new JButton("Search");

	String query = new String();
	String[] data = new String[300];
	/**
	 * Set up the mySQL database access to the treasurebox database.
	 */
	static String url = "jdbc:mysql://localhost:3306/treasurebox";
	static String driver = "com.mysql.jdbc.Driver";
	static String user = "treasureboxuser";
	static String pass = "treasure";

	boolean errFlag = false;

	public void focusGained(FocusEvent e) {
		if (searchText.getText().equals("Enter Search Text Here: "))
			searchText.setText("");
	}

	/**
	 * setSearchTypes - Reads the items table and creates the complete set of types for the current project. The types
	 * are used to populate the pulldouwn menu in the Search box.
	 */
	public void setSearchTypes() {
		types.removeAllItems();
		boolean noneFlag = false;
		Connection conn = null;
		try {
			Class.forName(driver).newInstance();
			conn = DriverManager.getConnection(url, user, pass);
			Statement stmt = conn.createStatement();
			ResultSet rs;
			String query = "SELECT DISTINCT(type) FROM items WHERE projectKey = " + projectKey + ";";
			if (finder.logFlag)
				Finder.logMessage(3, query);
			rs = stmt.executeQuery(query);
			while (rs.next()) {
				String type = rs.getString("type");
				if (!type.equals("ROOT"))
					types.addItem(type);
				if (type.equals("None"))
					noneFlag = true;
			}
			if (!noneFlag)
				types.addItem("None");
			types.setSelectedItem("None");
			conn.close();
		} catch (Exception e1) {
			if (finder.logFlag)
				Finder.logMessage(2, "Database error when getting types for the Search Panel.");
		}
	}

	public void focusLost(FocusEvent e) {
	}

	/**
	 * This is the main constructor for the application. It sets up the main display after prompting the user for a
	 * specific project. All the records for that project are loaded from the database to build the tree panel. After
	 * the tree is built the system selects the top of the tree displaying the project information and an image if one
	 * has been added. Any item(s) added directly under the selected item are displayed in the list panel. Items can be
	 * selected from either the tree or list panel. Search functionality is provided with the search panel on the top
	 * right of the display. All items which match the specified search criteria are displayed in the list panel.
	 * 
	 * Item management and Photo management are handled by the Finder component.
	 * 
	 */
	public TreasureBox() {
		super("Treasure Box - Item Organizer");
		getRootPane().setDefaultButton(searchButton);
		Border blackline;
		blackline = BorderFactory.createLineBorder(Color.black);
		searchText.addFocusListener(this);

		ActionListener al = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String searchTextSQL = null;
				String searchTypeSQL = null;
				String searchLocation = null;
				String command = e.getActionCommand();
				/**
				 * Support for the Search component. Builds the SQL statement corresponding to parameters entered on the
				 * screen. *
				 */
				if (command.equals("search")) {
					/**
					 * If the ROOT record for the project is selected set the search location to the ROOT name.
					 * Otherwise set the search location to the item location plus the item name. This provides a
					 * localized search of the selected item. All returned items are below the selected item in the
					 * tree. If the ROOT record is selected a global search of the tree is performed.
					 */
					if (finder.selectedLocation.equals(" ")) {
						searchLocation = finder.selectedName;
					} else {
						searchLocation = finder.selectedLocation + "|" + finder.selectedName;
					}
					searchTextSQL = searchText.getText();
					searchTypeSQL = types.getSelectedItem().toString();

					/**
					 * Use the search radio button to build the correct select statement. The search is either by Name,
					 * Description or Both Name and Description.
					 */
					if (nameButton.isSelected()) {
						query = "SELECT name, description, location FROM items WHERE location LIKE '" + searchLocation
								+ "%' AND name LIKE '%" + searchTextSQL + "%'";
					} else if (descButton.isSelected()) {
						query = "SELECT name, description, location FROM items WHERE location LIKE '" + searchLocation
								+ "%' AND description LIKE '%" + searchTextSQL + "%'";
					} else {
						query = "SELECT name, description, location FROM items WHERE location LIKE '" + searchLocation
								+ "%' AND ((name LIKE '%" + searchTextSQL + "%') OR (description LIKE '%"
								+ searchTextSQL + "%'))";
					}
					/**
					 * If the type combo-box has a selected value and the value does not equal "None" add the specified
					 * type value to the search criteria.
					 */
					if (!(types.getSelectedIndex() == -1) && !searchTypeSQL.equals("None")) {
						query = query + " AND type = '" + searchTypeSQL + "'";
					}
					query = query + " AND projectKey = " + projectKey + " ORDER BY location, itemOrder;";
					if (finder.logFlag)
						Finder.logMessage(3, query);
					int count = 1;
					int lnCnt = 0;
					int llCnt = 0;
					int addblanks = 0;
					data[0] = "Name                " + "Description              " + ""
							+ "                                    Location";
					for (int i = 1; i < 300; i++)
						data[i] = "";
					/**
					 * Once the SQL search statement is build execute it and place the returned items into the list
					 * panel. The Name field has spaces added to it to make the Description line up. Only the first 60
					 * characters of the Description field are displayed in the list.
					 */
					Connection conn = null;
					try {
						Class.forName(driver).newInstance();
						conn = DriverManager.getConnection(url, user, pass);
						Statement stmt = conn.createStatement();
						ResultSet rs;

						rs = stmt.executeQuery(query);

						while (rs.next()) {
							String Name = rs.getString("Name");
							displayListName[lnCnt++] = Name;
							addblanks = 20 - Name.length();
							StringBuffer NameSB = new StringBuffer(Name);
							for (int i = 0; i < addblanks; i++)
								NameSB.append(" ");
							Name = new String(NameSB);

							String Description = rs.getString("description");
							addblanks = 60 - Description.length();
							StringBuffer DescriptionSB = new StringBuffer(Description);
							if (DescriptionSB.length() > 60) {
								DescriptionSB = new StringBuffer(DescriptionSB.substring(0, 59));
								addblanks = 1;
							}
							for (int i = 0; i <= addblanks; i++)
								DescriptionSB.append(" ");
							Description = new String(DescriptionSB);

							String Location = rs.getString("location");
							displayListLocation[llCnt++] = Location;
							data[count] = Name + Description + Location;
							count++;
						}
						conn.close();
					} catch (Exception e1) {
						if (finder.logFlag)
							Finder.logMessage(2, "Error when performing a search query.");
					}
					/**
					 * Take the search data and place it in the displayList
					 */
					String[] data2 = new String[count];
					for (int i = 0; i < count; i++)
						data2[i] = data[i];
					displayList = new JList(data2);
					Font displayFont = new Font("Courier", Font.BOLD, 12);
					displayList.setFont(displayFont);

					listView.getViewport().setView(displayList);
					listView.setBackground(Color.orange);
					listView.setForeground(Color.orange.brighter());
				}
				displayList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
				displayList.addListSelectionListener(listSelectionListener);
			}
		};
		/**
		 * When the user selects an item from the list
		 */
		listSelectionListener = new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent listSelectionEvent) {
				JList list = (JList) listSelectionEvent.getSource();
				int selection = list.getSelectedIndex();
				finder.selectedName = displayListName[selection + 1];
				finder.selectedLocation = displayListLocation[selection + 1];
				Object selectionValue = list.getSelectedValue();
				// Get the name and location from the selected item in the list.
				String name = selectionValue.toString().substring(0, 20).trim();
				String location = selectionValue.toString().substring(81);
				DefaultMutableTreeNode node = convertLocationToTreeNode(location + "|" + name);
				DefaultMutableTreeNode parent;
				TreePath path = new TreePath(node.getPath());
				if (node.isLeaf()) {
					parent = (DefaultMutableTreeNode) node.getParent();
					expandNode(new TreePath(parent.getPath()));
				} else {
					expandNode(path);
				}
				finder.getTree().setSelectionPath(path);
			}

			private void expandNode(TreePath parent) {
				TreeNode node = (TreeNode) parent.getLastPathComponent();
				if (node.getChildCount() >= 0) {
					Enumeration<?> e = node.children();
					while (e.hasMoreElements()) {
						TreeNode n = (TreeNode) e.nextElement();
						TreePath path = parent.pathByAddingChild(n);
						expandNode(path);
					}
				}
				finder.getTree().expandPath(parent);
			}
		};

		buildMenuBar();

		displayList = new JList();

		top = finder.getSplitPane();
		top.setBorder(blackline);

		finder.tree.addTreeSelectionListener(this);
		listView.getViewport().setView(displayList);

		/**
		 * Create a vertical split pane for TreasereBox. In the top part of the split pane place the horizontal split
		 * pane from finder which contains the item tree and the item display side by side. The bottom of this split
		 * pane will contain a list box of items. The items will either be the children items of the selected tree item
		 * or the items which match the search criteria when the search button is pressed.
		 */
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, top, listView);
		splitPane.setOneTouchExpandable(true);
		splitPane.setDividerLocation(450);
		splitPane.setBackground(Color.yellow);

		JPanel search = new JPanel(new BorderLayout());
		JPanel searchBox = new JPanel(new FlowLayout());
		JPanel searchType = new JPanel(new BorderLayout());

		// Create the radio buttons.
		nameButton = new JRadioButton("Name");
		nameButton.setMnemonic(KeyEvent.VK_N);
		nameButton.setActionCommand("name");
		nameButton.setSelected(true);

		descButton = new JRadioButton("Description");
		descButton.setMnemonic(KeyEvent.VK_D);
		descButton.setActionCommand("description");

		bothButton = new JRadioButton("Both");
		bothButton.setMnemonic(KeyEvent.VK_B);
		bothButton.setActionCommand("both");

		// Group the radio buttons.
		ButtonGroup group = new ButtonGroup();
		group.add(nameButton);
		group.add(descButton);
		group.add(bothButton);

		// Register a listener for the radio buttons.
		nameButton.addActionListener(null);
		descButton.addActionListener(null);
		bothButton.addActionListener(null);

		// Put the radio buttons into a panel.
		JPanel radioPanel = new JPanel(new FlowLayout());
		radioPanel.setBorder(blackline);
		radioPanel.add(nameButton);
		radioPanel.add(descButton);
		radioPanel.add(bothButton);

		// Set up the types for the selected project.
		JPanel searchTextType = new JPanel(new BorderLayout());
		types = new JComboBox();
		types.setEditable(false);
		setSearchTypes();

		// Search Type Label
		JTextField itemTypeLabel = new JTextField();
		itemTypeLabel.setFont(new Font("TimesNewRoman", Font.PLAIN, 12));
		itemTypeLabel.setText("Select the Item Type or \"None\"");

		// Add search Type Label and the types combobox to the searchType panel
		searchType.add(itemTypeLabel, BorderLayout.NORTH);
		searchType.add(types, BorderLayout.CENTER);
		// Add the search text and search type panel to searchTextType panel
		searchTextType.add(searchText, BorderLayout.WEST);
		searchTextType.add(searchType, BorderLayout.EAST);
		searchTextType.add(radioPanel, BorderLayout.SOUTH);

		// Combine searchTextType and searchButton into searchBox panel
		searchBox.add(searchTextType);
		searchBox.add(searchButton);
		searchBox.setBackground(Color.green.darker());
		searchBox.setBorder(blackline);

		// Set up action listener for the search button
		searchButton.addActionListener(al);
		searchButton.setActionCommand("search");

		// Create the text area for the Location
		displayLocation = new JTextArea(3, 0);
		displayLocation.setBackground(Color.yellow);
		displayLocation.setBorder(blackline);
		displayLocation.setLineWrap(true);
		displayLocation.setWrapStyleWord(false);
		// Place the Location text into the scroll pane locationView
		locationView = new JScrollPane(displayLocation);
		locationView.setBorder(blackline);

		search.add(locationView, BorderLayout.WEST);
		search.add(searchBox, BorderLayout.EAST);
		search.setBackground(Color.gray.brighter());
		search.setBorder(blackline);
		search.setMinimumSize(new Dimension(400, 80));

		// Add the split pane to this frame
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(search, BorderLayout.NORTH);
		getContentPane().add(splitPane, BorderLayout.CENTER);

		TreePath path = new TreePath(finder.treeTop);
		finder.getTree().setSelectionPath(path);
	}

	/**
	 * TreasureBox listens for when the user selects a new item in the tree panel of the finder component. The Finder
	 * class handles the tree and image panels but TreasureBox is responsible for updating the list panel in the bottom
	 * of its own split pane. The SQL query returns and displays any items which are directly below the selected item
	 * (i.e. children of the selected item). The list of item names and locations in the list box are also maintained in
	 * two arrays for ease of processing.
	 */
	public void valueChanged(TreeSelectionEvent e) {
		int locationSize = 0;
		data = new String[300];
		locationView.setSize(425, 80);
		locationView.setPreferredSize(new Dimension(425, 80));

		TreePath tp = finder.getTree().getSelectionPath();
		if (tp != null) {
			DefaultMutableTreeNode node = ((DefaultMutableTreeNode) tp.getLastPathComponent());
			if (node == null)
				return;
			Object nodeInfo = node.getUserObject();
			// Returns the number of levels above the specified node.
			int pathCnt = tp.getPathCount() - 1;
			StringBuffer sbWhere = new StringBuffer();
			// Build Location value and size to compare
			for (int i = 0; i < pathCnt; i++) {
				sbWhere.append(tp.getPathComponent(i));
				sbWhere.append("|");
			}
			sbWhere.append(nodeInfo.toString());
			loc = sbWhere.toString();
			displayLocation.setText("Location: " + loc);
			displayLocation.setCaretPosition(0);
			locationSize = sbWhere.length();
			query = "SELECT name, location, description FROM items WHERE locationSize = " + locationSize
					+ " AND location LIKE '" + loc + "%' AND projectKey = " + finder.projectKey
					+ " ORDER BY itemOrder;";
			if (finder.logFlag)
				Finder.logMessage(3, query);
			// SQL connection and execution
			int count = 1;
			Connection conn = null;
			try {
				conn = DriverManager.getConnection(url, user, pass);
				Statement stmt = conn.createStatement();
				ResultSet rs;

				rs = stmt.executeQuery(query);
				int lnCnt = 0;
				int llCnt = 0;
				int addblanks = 0;

				data[0] = "Name                " + "Description              " + ""
						+ "                                    Location";
				while (rs.next()) {
					String Name = rs.getString("Name");
					displayListName[lnCnt++] = Name;
					addblanks = 20 - Name.length();
					StringBuffer NameSB = new StringBuffer(Name);
					for (int i = 0; i < addblanks; i++)
						NameSB.append(" ");
					Name = new String(NameSB);

					String Description = rs.getString("Description");
					addblanks = 60 - Description.length();
					StringBuffer DescriptionSB = new StringBuffer(Description);
					if (DescriptionSB.length() > 60) {
						DescriptionSB = new StringBuffer(DescriptionSB.substring(0, 59));
						addblanks = 1;
					}
					for (int i = 0; i <= addblanks; i++)
						DescriptionSB.append(" ");
					Description = new String(DescriptionSB);

					String Location = rs.getString("Location");
					displayListLocation[llCnt++] = Location;
					data[count] = Name + Description + Location;
					count++;
				}
				conn.close();
			} catch (Exception e1) {
				if (finder.logFlag)
					Finder.logMessage(2, "Error getting items the for List Panel when an item is selected in the tree");
			}

			if (finder.getTree().isSelectionEmpty()) {
				data[0] = "Select a Node in the Tree Or Search";
				displayList = new JList(data);
				Font displayFont = new Font("Courier", Font.BOLD, 12);
				displayList.setFont(displayFont);
				listView.getViewport().setView(displayList);
			} else {
				String[] data2 = new String[count];
				for (int i = 0; i < count; i++)
					data2[i] = data[i];

				displayList = new JList(data2);
				Font displayFont = new Font("Courier", Font.BOLD, 12);
				displayList.setFont(displayFont);
				listView.getViewport().setView(displayList);
			}
			displayList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			displayList.addListSelectionListener(listSelectionListener);
			finder.getTree().setExpandsSelectedPaths(true);
			finder.getTree().setSelectionPath(tp);
			finder.getTree().repaint();
		}
	}

	/**
	 * Builds the menu bar. Currently supporting a File, Item, Tools and Help at the top level.
	 */
	public void buildMenuBar() {
		JMenu menu;
		JMenuItem menuItem;

		// File menu
		menu = new JMenu("File");
		menu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(menu);

		menuItem = new JMenuItem("Manage Projects...");
		menuItem.setMnemonic(KeyEvent.VK_P);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				int rtn = finder.createNodes(true);
				if ((rtn == 0)) {
					projectKey = finder.projectKey;
					finder.model.setRoot(finder.treeTop);
					TreePath path = new TreePath(finder.treeTop);
					finder.tree.setSelectionPath(path);
					finder.tree.validate();
					finder.model.reload(finder.treeTop);
					finder.expandAll(finder.tree, true);
					finder.tree.setSelectionPath(path);
					setSearchTypes();
				}
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Toggle Maintenance Mode");
		menuItem.setMnemonic(KeyEvent.VK_T);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if (finder.maintenanceMode) {
					finder.maintenanceMode = false;
					finder.addItemButton.setVisible(false);
					finder.editItemButton.setVisible(false);
					finder.deleteItemButton.setVisible(false);
					finder.itemButtonPanel.setVisible(false);
					finder.treeView.setPreferredSize((new Dimension(220, finder.splitPane.getHeight() - 10)));
				} else {
					finder.maintenanceMode = true;
					finder.addItemButton.setVisible(true);
					finder.editItemButton.setVisible(true);
					finder.deleteItemButton.setVisible(true);
					finder.itemButtonPanel.setVisible(true);
					finder.treeView.setPreferredSize((new Dimension(220, finder.splitPane.getHeight() - 60)));
				}
				DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) finder.tree
						.getLastSelectedPathComponent();
				finder.tree.clearSelection();
				TreePath path = new TreePath(selectedNode.getPath());
				finder.tree.setSelectionPath(path);
				finder.treeView.repaint();
				setSearchTypes();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Windows Look and Feel");
		menuItem.setMnemonic(KeyEvent.VK_W);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				try {
					UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
					SwingUtilities.updateComponentTreeUI(frame);
					frame.pack();
				} catch (Exception e) {
					if (finder.logFlag)
						Finder.logMessage(2, "Error when setting Windows Look and Feel");
				}
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Java Look and Feel");
		menuItem.setMnemonic(KeyEvent.VK_J);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_J, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				try {
					UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
					SwingUtilities.updateComponentTreeUI(frame);
					frame.pack();
				} catch (Exception e) {
					if (finder.logFlag)
						Finder.logMessage(2, "Error when setting Java Look and Feel");
				}
				SwingUtilities.updateComponentTreeUI(frame);
				frame.pack();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Motif Look and Feel");
		menuItem.setMnemonic(KeyEvent.VK_M);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				try {
					UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
					SwingUtilities.updateComponentTreeUI(frame);
					frame.pack();
				} catch (Exception e) {
					if (finder.logFlag)
						Finder.logMessage(2, "Error when setting Motif Look and Feel");
				}
			}
		});

		menu.add(menuItem);
		menuItem = new JMenuItem("Exit");
		menuItem.setMnemonic(KeyEvent.VK_E);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				System.exit(0);
			}
		});
		menu.add(menuItem);

		// Item Menu
		menu = new JMenu("Item");
		menu.setMnemonic(KeyEvent.VK_I);
		menuBar.add(menu);

		menuItem = new JMenuItem("Full Screen Image");
		menuItem.setMnemonic(KeyEvent.VK_S);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				MaxScreen ms = new MaxScreen(installPath + finder.projectName + "/" + finder.selectedImage);
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Reorder...");
		menuItem.setMnemonic(KeyEvent.VK_R);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				reorderItem();
			}
		});
		menu.add(menuItem);

		// Reports Menu
		menu = new JMenu("Tools");
		menu.setMnemonic(KeyEvent.VK_T);
		menuBar.add(menu);

		menuItem = new JMenuItem("Optimize Screen");
		menuItem.setMnemonic(KeyEvent.VK_O);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if (finder.maintenanceMode) {
					finder.itemButtonPanel.setVisible(false);
					finder.itemButtonPanel.setVisible(true);
					finder.treeView.setPreferredSize((new Dimension(220, finder.splitPane.getHeight() - 60)));
				} else {
					finder.itemButtonPanel.setVisible(true);
					finder.itemButtonPanel.setVisible(false);
					finder.treeView.setPreferredSize((new Dimension(220, finder.splitPane.getHeight() - 10)));
				}
				DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) finder.tree
						.getLastSelectedPathComponent();
				finder.tree.clearSelection();
				TreePath path = new TreePath(selectedNode.getPath());
				finder.tree.setSelectionPath(path);
				finder.treeView.repaint();
				setSearchTypes();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Print Current Image...");
		menuItem.setMnemonic(KeyEvent.VK_I);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				printImage();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Print Current Item...");
		menuItem.setMnemonic(KeyEvent.VK_C);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				printItem();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("New Trace File");
		menuItem.setMnemonic(KeyEvent.VK_N);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if (!finder.logFlag) {
					try {
						finder.fstream = new FileWriter(finder.installPath + "treasureboxLog.txt");
					} catch (IOException e) {
						Finder.logMessage(2, "Got an file error when trying to create treasureboxLog.txt.");
					}
					Finder.msgWriter = new BufferedWriter(finder.fstream);
					finder.logFlag = true;
				}
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Save Trace File to View");
		menuItem.setMnemonic(KeyEvent.VK_S);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				finder.logFlag = false;
				try {
					Finder.msgWriter.close();
				} catch (IOException e) {
					Finder.logMessage(2, "Got an file error when trying to close treasureboxLog.txt.");
				}
			}
		});
		menu.add(menuItem);

		// Help Menu
		menu = new JMenu("Help");
		menu.setMnemonic(KeyEvent.VK_H);
		menuBar.add(menu);

		menuItem = new JMenuItem("Help...");
		menuItem.setMnemonic(KeyEvent.VK_H);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				displayHelp();
			}
		});
		menu.add(menuItem);
		this.setJMenuBar(menuBar);
	}

	/**
	 * Method: convertLocationToTreeNode() : An item's location represents its position in the item tree structure. In
	 * the database it is stored as a variable length String containing a concatenation of item names from the ROOT
	 * (project) node to the parent of the selected item. The bar(|) symbol is used as a separator between item names.
	 * 
	 * @param location
	 * @return
	 */
	public DefaultMutableTreeNode convertLocationToTreeNode(String location) {
		DefaultMutableTreeNode parent = finder.treeTop;
		int levelCount = 1;
		int fromIndex = 0;
		// Count the number of nodes
		while (location.indexOf("|", fromIndex) != -1) {
			levelCount++;
			fromIndex = location.indexOf("|", fromIndex) + 1;
		}
		int beginIndex = 0;
		int endIndex = -1;
		// Create an array of objects for the number of nodes and put the node names into the array.
		Object[] obj = new Object[levelCount + 1];
		for (int i = 0; i < levelCount; i++) {
			beginIndex = endIndex + 1;
			endIndex = location.indexOf("|", beginIndex);
			if (endIndex == -1)
				obj[i] = new DefaultMutableTreeNode(location.substring(beginIndex));
			else {
				obj[i] = new DefaultMutableTreeNode(location.substring(beginIndex, endIndex));
			}
		}
		// Find the item location in the tree.
		for (int i = 1; i < levelCount; i++) {
			int childCnt = parent.getChildCount();
			for (int j = 0; j < childCnt; j++) {
				if (parent.getChildAt(j).toString().equals(obj[i].toString())) {
					parent = (DefaultMutableTreeNode) parent.getChildAt(j);
					break;
				}
			}
		}
		return (parent);
	}

	public void displayHelp() {
		new HelpDialog(frame);
		return;
	}

	/**
	 * Method: reorderItem - Items which have more than one child can have these children rearranged in any order the
	 * user desires. By default the items are displayed in the order they were entered. If the user requests (through
	 * the menu) the child items under a selected item be reordered and there is more than a single child item a Reorder
	 * Panel will be displayed allowing the user to reorder the child items. Once the user is satisfied with the new
	 * order and selects OK the tree is redrawn and the database records are updated to reflect the new order.
	 */
	public static void reorderItem() {
		String[] children = new String[50];
		int[] order;
		TreeNode parent = (TreeNode) finder.getTree().getSelectionPath().getLastPathComponent();
		int childCount = parent.getChildCount();
		if (childCount < 2) {
			JOptionPane.showMessageDialog(null, "Need more than " + childCount + " to reorder!", "Cannot Reorder: "
					+ parent.toString(), JOptionPane.ERROR_MESSAGE);
			return;
		}
		for (int i = 0; i < childCount; i++) {
			TreeNode childNode = parent.getChildAt(i);
			children[i] = childNode.toString();
		}
		order = new int[childCount];
		ReorderDialog rd = new ReorderDialog(frame, true, children, childCount);
		order = rd.getOrder();
		if (rd.getAnswer()) {
			if (finder.logFlag)
				Finder.logMessage(1, " Reordering...");
			String updateStatement = new String();
			// SQL execution
			for (int i = 0; i < childCount; i++) {
				if (i != order[i]) {
					if (finder.selectedType.equals("ROOT")) {
						updateStatement = "UPDATE items SET itemOrder = " + i + " WHERE itemOrder = " + order[i]
								+ " AND name = '" + children[i] + "' AND location = '" + finder.selectedName
								+ "' AND projectKey = " + projectKey + ";";
					} else {
						updateStatement = "UPDATE items SET itemOrder = " + i + " WHERE itemOrder = " + order[i]
								+ " AND name = '" + children[i] + "' AND location = '" + finder.selectedLocation + "|"
								+ finder.selectedName + "' AND projectKey = " + projectKey + ";";
					}
					if (finder.logFlag)
						Finder.logMessage(3, updateStatement);
					Connection conn = null;
					try {
						Class.forName(driver).newInstance();
						conn = DriverManager.getConnection(url, user, pass);
						Statement stmt = conn.createStatement();
						stmt.executeUpdate(updateStatement);
						conn.close();
					} catch (Exception e1) {
						if (finder.logFlag)
							Finder.logMessage(2, "Database error when reordering items");
					}
				}
			}
			finder.createNodes(false);
			finder.model.setRoot(finder.treeTop);
			finder.model.reload(finder.treeTop);
			TreePath path = new TreePath(finder.treeTop);
			finder.tree.setSelectionPath(path);
			finder.expandAll(finder.tree, true);
			finder.tree.validate();
		}
	}

	public static void printImage() {
		try {
			performImagePrint();
		} catch (Exception e) {
			Finder.logMessage(2, "Got an exception while trying to print in performPrint().");
		}
	}

	public static void performImagePrint() throws Exception {

		String filename = installPath + finder.projectName + "/" + finder.itemPhotos[finder.currentPhoto];

		PrintRequestAttributeSet pras = new HashPrintRequestAttributeSet();
		DocFlavor flavor = DocFlavor.INPUT_STREAM.JPEG;
		PrintService printService[] = PrintServiceLookup.lookupPrintServices(flavor, pras);
		PrintService defaultService = PrintServiceLookup.lookupDefaultPrintService();
		PrintService service = ServiceUI.printDialog(null, 200, 200, printService, defaultService,
				DocFlavor.INPUT_STREAM.JPEG, pras);
		if (service != null) {
			DocPrintJob job = service.createPrintJob();
			PrintJobListener listener = new PrintJobAdapter() {
				public void printDataTransferCompleted(PrintJobEvent e) {
					if (finder.logFlag)
						Finder.logMessage(1, "Print Data Transfer Complete");
				}
			};
			job.addPrintJobListener(listener);
			FileInputStream fis = new FileInputStream(filename);
			DocAttributeSet das = new HashDocAttributeSet();
			Doc doc = new SimpleDoc(fis, flavor, das);
			job.print(doc, pras);
			Thread.sleep(10000);
		}
	}

	public void printItem() {
		try {
			PrinterJob job = PrinterJob.getPrinterJob();
			job.setPrintable(this);
			boolean ok = job.printDialog();
			if (ok) {
				try {
					job.print();
				} catch (PrinterException ex) {
					if (finder.logFlag)
						Finder.logMessage(2, "The print job did not complete successfully.");
				}
			}
		} catch (Exception e) {
			if (finder.logFlag)
				Finder.logMessage(2, "Got an exception while trying to print...");
		}
	}

	public int print(Graphics g, PageFormat pf, int page) throws PrinterException {
		if (page > 0) {
			return NO_SUCH_PAGE;
		}
		String printLine = null;
		Graphics2D g2d = (Graphics2D) g;
		g2d.translate(pf.getImageableX(), pf.getImageableY());
		AffineTransform at = new AffineTransform();
		at.translate(0, 0);
		int y = 0;
		g2d.drawString("Treasure Box Item Report", 210, y += g.getFontMetrics().getHeight());
		g2d.drawString(getDateTime(), 210, y += g.getFontMetrics().getHeight());
		g2d.drawString("Project Name: " + finder.projectName, 20, y += g.getFontMetrics().getHeight());
		g2d.drawString("Item Name: " + finder.selectedName, 30, y += g.getFontMetrics().getHeight());
		g2d.drawString("Item Type: " + finder.selectedType, 30, y += g.getFontMetrics().getHeight());
		g2d.drawString("Item Container: " + finder.selectedContainer, 30, y += g.getFontMetrics().getHeight());
		g2d.drawString("Item Description:", 30, y += g.getFontMetrics().getHeight());
		for (String line : finder.selectedDescription.split(newline)) {
			if (line.length() > 90) {
				int pieces = (int) line.length() / 90;
				StringBuffer parseLine = new StringBuffer(line);
				for (int i = 0; i < pieces; i++) {
					printLine = parseLine.substring(0, 90);
					g.drawString(printLine, 50, y += g.getFontMetrics().getHeight());
					parseLine = new StringBuffer(parseLine.substring(90));
				}
				if (parseLine.length() > 0)
					g.drawString(parseLine.toString(), 50, y += g.getFontMetrics().getHeight());
			} else
				g.drawString(line.toString(), 50, y += g.getFontMetrics().getHeight());
		}
		if (finder.photoCount > 0) {
			g2d.drawString("Item Images:", 30, y += g.getFontMetrics().getHeight());
			y += g.getFontMetrics().getHeight();
			for (int i = 0; i < finder.photoCount; i++) {
				String filename = installPath + finder.projectName + "/" + finder.itemPhotos[i];
				File file = new File(filename);
				Image printImage = null;
				try {
					printImage = ImageIO.read(file);
				} catch (IOException e) {
					if (finder.logFlag)
						Finder.logMessage(2, "Got an exception while trying to print the Item Report...");
				}
				g2d.drawImage(printImage, 30 + 110 * i, y, 100, 100, null);
			}
		}
		// We need to scale the image properly so that it fits on one page.
		double xScale = pf.getImageableWidth() / frame.getWidth();
		double yScale = pf.getImageableHeight() / frame.getHeight();

		// Maintain the aspect ratio by taking the min of those 2 factors and
		// using it to scale both dimensions.
		double aspectScale = Math.min(xScale, yScale);
		g2d.scale(aspectScale, aspectScale);
		g2d.drawRenderedImage((RenderedImage) frame.getIconImage(), at);
		/* Print the entire visible contents of a java.awt.Frame */
		// frame.printAll(g2d);
		return PAGE_EXISTS;
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be invoked from the event-dispatching thread.
	 */
	private static void createAndShowGUI() {
		// Create and set up the window.
		frame = new TreasureBox();
		frame.setIconImage(Toolkit.getDefaultToolkit().getImage(installPath + "treasurebox.png"));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			SwingUtilities.updateComponentTreeUI(frame);
			frame.pack();
			if (finder.logFlag)
				Finder.logMessage(1, UIManager.getSystemLookAndFeelClassName());
		} catch (Exception f) {
			if (finder.logFlag)
				Finder.logMessage(2, "ClassNotFoundException when setting Look and Feel.");
		}
		frame.setVisible(true);
	}

	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}
}

class SQLfilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
		return f.isDirectory() || f.getName().toLowerCase().endsWith(".jpg");
	}

	public String getDescription() {
		return "jpg files";
	}
}
